@@ -37,7 +37,7 @@
 #include "auto_mem.hh"
 #include "config.h"
 #include "lnav_log.hh"
-#include "pcrepp/pcrepp.hh"
+#include "pcrepp/pcre2pp.hh"
 
 attr_line_t&
 attr_line_t::with_ansi_string(const char* str, ...)
@@ -91,18 +91,19 @@ using chunk = mapbox::util::variant<word, space, corrupt, eof>;
 chunk
 consume(const string_fragment text)
 {
-    static const pcrepp WORD_RE(R"((*UTF)^[^\p{Z}\p{So}\p{C}]+)");
-    static const pcrepp SPACE_RE(R"((*UTF)^\s)");
+    static const auto WORD_RE
+        = lnav::pcre2pp::code::from_const(R"((*UTF)^[^\p{Z}\p{So}\p{C}]+)");
+    static const auto SPACE_RE
+        = lnav::pcre2pp::code::from_const(R"((*UTF)^\s)");
 
     if (text.empty()) {
         return eof{text};
     }
 
-    pcre_input pi(text);
-    pcre_context_static<30> pc;
-
-    if (WORD_RE.match(pc, pi, PCRE_NO_UTF8_CHECK)) {
-        auto split_res = text.split_n(pc.all()->length()).value();
+    auto word_find_res
+        = WORD_RE.find_in(text, PCRE2_NO_UTF_CHECK).ignore_error();
+    if (word_find_res) {
+        auto split_res = text.split_n(word_find_res->f_all.length()).value();
 
         return word{split_res.first, split_res.second};
     }
@@ -113,8 +114,10 @@ consume(const string_fragment text)
         return space{split_res.first, split_res.second};
     }
 
-    if (SPACE_RE.match(pc, pi, PCRE_NO_UTF8_CHECK)) {
-        auto split_res = text.split_n(pc.all()->length()).value();
+    auto space_find_res
+        = SPACE_RE.find_in(text, PCRE2_NO_UTF_CHECK).ignore_error();
+    if (space_find_res) {
+        auto split_res = text.split_n(space_find_res->f_all.length()).value();
 
         return space{split_res.first, split_res.second};
     }
@@ -184,8 +187,6 @@ attr_line_t::insert(size_t index,
         return *this;
     }
 
-    static const pcrepp SPACE_RE(R"(\s?)");
-
     auto starting_line_index = this->al_string.rfind('\n', index);
     if (starting_line_index == std::string::npos) {
         starting_line_index = 0;
@@ -199,7 +200,9 @@ attr_line_t::insert(size_t index,
         this->al_string, starting_line_index, this->al_string.length());
     string_fragment last_word;
     ssize_t line_ch_count = 0;
+    ssize_t line_indent_count = 0;
     auto needs_indent = false;
+    auto last_was_pre = false;
 
     while (!text_to_wrap.empty()) {
         if (needs_indent) {
@@ -213,6 +216,7 @@ attr_line_t::insert(size_t index,
             split_attrs(*this, indent_lr);
             indent_lr.lr_end += tws->tws_padding_indent;
             line_ch_count += tws->tws_padding_indent;
+            line_indent_count += tws->tws_padding_indent;
             if (!indent_lr.empty()) {
                 this->al_attrs.emplace_back(indent_lr, SA_PREFORMATTED.value());
             }
@@ -221,18 +225,34 @@ attr_line_t::insert(size_t index,
                 tws->tws_indent + tws->tws_padding_indent);
             needs_indent = false;
         }
-        auto chunk = text_stream::consume(text_to_wrap);
 
-        text_to_wrap = chunk.match(
+        text_stream::chunk next_chunk(mapbox::util::no_init{});
+        auto pre_iter = find_string_attr_containing(
+            this->al_attrs, &SA_PREFORMATTED, text_to_wrap.sf_begin);
+        if (pre_iter != this->al_attrs.end()) {
+            auto pre_len = pre_iter->sa_range.lr_end - text_to_wrap.sf_begin;
+            auto pre_lf = text_to_wrap.find('\n');
+            if (pre_lf && pre_lf.value() < pre_len) {
+                pre_len = pre_lf.value() + 1;
+            }
+
+            auto pre_pair = text_to_wrap.split_n(pre_len);
+            next_chunk = text_stream::word{
+                pre_pair->first,
+                pre_pair->second,
+            };
+        }
+        if (!next_chunk.valid()) {
+            next_chunk = text_stream::consume(text_to_wrap);
+        }
+
+        text_to_wrap = next_chunk.match(
             [&](text_stream::word word) {
                 auto ch_count
                     = word.w_word.utf8_length().unwrapOr(word.w_word.length());
 
-                if ((line_ch_count + ch_count) > usable_width
-                    && find_string_attr_containing(this->al_attrs,
-                                                   &SA_PREFORMATTED,
-                                                   text_to_wrap.sf_begin)
-                        == this->al_attrs.end())
+                if (line_ch_count > line_indent_count && !last_was_pre
+                    && (line_ch_count + ch_count) > usable_width)
                 {
                     this->insert(word.w_word.sf_begin, 1, '\n');
                     this->insert(word.w_word.sf_begin + 1,
@@ -249,6 +269,7 @@ attr_line_t::insert(size_t index,
                                                     SA_PREFORMATTED.value());
                     }
                     line_ch_count = tws->tws_padding_indent + ch_count;
+                    line_indent_count = tws->tws_padding_indent;
                     auto trailing_space_count = 0;
                     if (!last_word.empty()) {
                         trailing_space_count
@@ -262,12 +283,18 @@ attr_line_t::insert(size_t index,
                                  1 + tws->tws_indent + tws->tws_padding_indent);
                 }
                 line_ch_count += ch_count;
+                if (word.w_word.endswith("\n")) {
+                    line_ch_count = 0;
+                    line_indent_count = 0;
+                    needs_indent = true;
+                }
 
                 return word.w_remaining;
             },
             [&](text_stream::space space) {
                 if (space.s_value == "\n") {
                     line_ch_count = 0;
+                    line_indent_count = 0;
                     needs_indent = true;
                     return space.s_remaining;
                 }
@@ -286,6 +313,7 @@ attr_line_t::insert(size_t index,
                                     space.s_value.length());
                         this->insert(space.s_value.sf_begin, "\n");
                         line_ch_count = 0;
+                        line_indent_count = 0;
                         needs_indent = true;
 
                         auto trailing_space_count = 0;
@@ -317,9 +345,10 @@ attr_line_t::insert(size_t index,
             [](text_stream::corrupt corrupt) { return corrupt.c_remaining; },
             [](text_stream::eof eof) { return eof.e_remaining; });
 
-        if (chunk.is<text_stream::word>()) {
+        if (next_chunk.is<text_stream::word>()) {
             last_word = text_to_wrap;
         }
+        last_was_pre = (pre_iter != this->al_attrs.end());
 
         ensure(this->al_string.data() == text_to_wrap.sf_string);
         ensure(text_to_wrap.sf_begin <= text_to_wrap.sf_end);
@@ -410,7 +439,7 @@ attr_line_t::apply_hide()
 
     for (auto& sattr : sa) {
         if (sattr.sa_type == &SA_HIDDEN && sattr.sa_range.length() > 3) {
-            struct line_range& lr = sattr.sa_range;
+            auto& lr = sattr.sa_range;
 
             std::for_each(sa.begin(), sa.end(), [&](string_attr& attr) {
                 if (attr.sa_type == &VC_STYLE && lr.contains(attr.sa_range)) {
@@ -428,7 +457,7 @@ attr_line_t::apply_hide()
 }
 
 attr_line_t&
-attr_line_t::rtrim()
+attr_line_t::rtrim(nonstd::optional<const char*> chars)
 {
     auto index = this->al_string.length();
 
@@ -439,7 +468,12 @@ attr_line_t::rtrim()
         {
             break;
         }
-        if (!isspace(this->al_string[index - 1])) {
+        if (chars
+            && strchr(chars.value(), this->al_string[index - 1]) == nullptr)
+        {
+            break;
+        }
+        if (!chars && !isspace(this->al_string[index - 1])) {
             break;
         }
     }
@@ -461,7 +495,9 @@ attr_line_t::erase(size_t pos, size_t len)
 
     this->al_string.erase(pos, len);
 
-    shift_string_attrs(this->al_attrs, pos, -((int32_t) len));
+    shift_string_attrs(this->al_attrs,
+                       line_range{(int) pos, (int) (pos + len)},
+                       -((int32_t) len));
     auto new_end = std::remove_if(
         this->al_attrs.begin(), this->al_attrs.end(), [](const auto& attr) {
             return attr.sa_range.empty();
@@ -504,6 +540,33 @@ line_range::intersection(const line_range& other) const
     return line_range{std::max(this->lr_start, other.lr_start), actual_end};
 }
 
+line_range&
+line_range::shift_range(const line_range& cover, int32_t amount)
+{
+    if (cover.lr_end <= this->lr_start) {
+        this->lr_start = std::max(0, this->lr_start + amount);
+        if (this->lr_end != -1) {
+            this->lr_end = std::max(0, this->lr_end + amount);
+        }
+    } else {
+        if (amount < 0 && cover.contains(*this)) {
+            this->lr_start = cover.lr_start;
+        }
+        if (this->lr_end != -1) {
+            if (cover.lr_start < this->lr_end) {
+                if (amount < 0 && amount < (cover.lr_start - this->lr_end)) {
+                    this->lr_end = cover.lr_start;
+                } else {
+                    this->lr_end
+                        = std::max(this->lr_start, this->lr_end + amount);
+                }
+            }
+        }
+    }
+
+    return *this;
+}
+
 line_range&
 line_range::shift(int32_t start, int32_t amount)
 {
@@ -534,3 +597,120 @@ line_range::shift(int32_t start, int32_t amount)
 
     return *this;
 }
+
+string_attrs_t::const_iterator
+find_string_attr(const string_attrs_t& sa, size_t near)
+{
+    auto nearest = sa.end();
+    ssize_t last_diff = INT_MAX;
+
+    for (auto iter = sa.begin(); iter != sa.end(); ++iter) {
+        const auto& lr = iter->sa_range;
+
+        if (!lr.is_valid() || !lr.contains(near)) {
+            continue;
+        }
+
+        ssize_t diff = near - lr.lr_start;
+        if (diff < last_diff) {
+            last_diff = diff;
+            nearest = iter;
+        }
+    }
+
+    return nearest;
+}
+
+void
+shift_string_attrs(string_attrs_t& sa, int32_t start, int32_t amount)
+{
+    for (auto& iter : sa) {
+        iter.sa_range.shift(start, amount);
+    }
+}
+
+void
+shift_string_attrs(string_attrs_t& sa, const line_range& cover, int32_t amount)
+{
+    for (auto& iter : sa) {
+        iter.sa_range.shift_range(cover, amount);
+    }
+}
+
+struct line_range
+find_string_attr_range(const string_attrs_t& sa, string_attr_type_base* type)
+{
+    auto iter = find_string_attr(sa, type);
+
+    if (iter != sa.end()) {
+        return iter->sa_range;
+    }
+
+    return line_range();
+}
+
+void
+remove_string_attr(string_attrs_t& sa, const line_range& lr)
+{
+    string_attrs_t::iterator iter;
+
+    while ((iter = find_string_attr(sa, lr)) != sa.end()) {
+        sa.erase(iter);
+    }
+}
+
+void
+remove_string_attr(string_attrs_t& sa, string_attr_type_base* type)
+{
+    for (auto iter = sa.begin(); iter != sa.end();) {
+        if (iter->sa_type == type) {
+            iter = sa.erase(iter);
+        } else {
+            ++iter;
+        }
+    }
+}
+
+string_attrs_t::iterator
+find_string_attr(string_attrs_t& sa, const line_range& lr)
+{
+    string_attrs_t::iterator iter;
+
+    for (iter = sa.begin(); iter != sa.end(); ++iter) {
+        if (lr.contains(iter->sa_range)) {
+            break;
+        }
+    }
+
+    return iter;
+}
+
+string_attrs_t::const_iterator
+find_string_attr(const string_attrs_t& sa,
+                 const string_attr_type_base* type,
+                 int start)
+{
+    string_attrs_t::const_iterator iter;
+
+    for (iter = sa.begin(); iter != sa.end(); ++iter) {
+        if (iter->sa_type == type && iter->sa_range.lr_start >= start) {
+            break;
+        }
+    }
+
+    return iter;
+}
+
+nonstd::optional<const string_attr*>
+get_string_attr(const string_attrs_t& sa,
+                const string_attr_type_base* type,
+                int start)
+{
+    auto iter = find_string_attr(sa, type, start);
+
+    if (iter == sa.end()) {
+        return nonstd::nullopt;
+    }
+
+    return nonstd::make_optional(&(*iter));
+}
