@@ -56,20 +56,14 @@
 #include <thread>
 #include <vector>
 
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
 #include <sys/param.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
 #include <sys/wait.h>
 
-#ifdef HAVE_PCRE_H
-#    include <pcre.h>
-#elif HAVE_PCRE_PCRE_H
-#    include <pcre/pcre.h>
-#else
-#    error "pcre.h not found?"
-#endif
-
 #if defined HAVE_NCURSESW_CURSES_H
 #    include <ncursesw/curses.h>
 #    include <ncursesw/termcap.h>
@@ -196,7 +190,7 @@ log_argv(int argc, char* argv[])
     const char* log_path = getenv("LNAV_LOG_PATH");
 
     if (log_path != nullptr) {
-        lnav_log_file = make_optional_from_nullable(fopen(log_path, "a"));
+        lnav_log_file = make_optional_from_nullable(fopen(log_path, "ae"));
     }
 
     log_info("argv[%d] =", argc);
@@ -215,14 +209,14 @@ void
 log_host_info()
 {
     char cwd[MAXPATHLEN];
-    const char* jittarget;
+    char jittarget[128];
     struct utsname un;
     struct rusage ru;
-    int pcre_jit;
+    uint32_t pcre_jit;
 
     uname(&un);
-    pcre_config(PCRE_CONFIG_JIT, &pcre_jit);
-    pcre_config(PCRE_CONFIG_JITTARGET, &jittarget);
+    pcre2_config(PCRE2_CONFIG_JIT, &pcre_jit);
+    pcre2_config(PCRE2_CONFIG_JITTARGET, jittarget);
 
     log_info("uname:");
     log_info("  sysname=%s", un.sysname);
@@ -398,6 +392,21 @@ log_msg_extra_complete()
     };
 }
 
+void
+log_backtrace(lnav_log_level_t level)
+{
+#ifdef HAVE_EXECINFO_H
+    int frame_count;
+    void* frames[128];
+
+    frame_count = backtrace(frames, 128);
+    auto bt = backtrace_symbols(frames, frame_count);
+    for (int lpc = 0; lpc < frame_count; lpc++) {
+        log_msg(level, __FILE__, __LINE__, "%s", bt[lpc]);
+    }
+#endif
+}
+
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-result"
 static void
