@@ -289,12 +289,14 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
             auto um
                 = lnav::console::user_message::error(
                       attr_line_t("unable to compile PRQL: ").append(stmt_al))
-                      .with_reason((std::string) msg.reason);
+                      .with_reason(
+                          attr_line_t::from_ansi_str((std::string) msg.reason));
             if (!msg.display.empty()) {
-                um.with_note((std::string) msg.display);
+                um.with_note(
+                    attr_line_t::from_ansi_str((std::string) msg.display));
             }
             for (const auto& hint : msg.hints) {
-                um.with_help(hint.data());
+                um.with_help(attr_line_t::from_ansi_str((std::string) hint));
                 break;
             }
             return Err(um);
@@ -463,14 +465,14 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         }
         lnav_data.ld_filter_view.reload_data();
         lnav_data.ld_files_view.reload_data();
-        lnav_data.ld_views[LNV_DB].reload_data();
-        lnav_data.ld_views[LNV_DB].set_left(0);
 
         lnav_data.ld_active_files.fc_files
             | lnav::itertools::for_each(&logfile::dump_stats);
         if (ec.ec_sql_callback != sql_callback) {
             retval = ec.ec_accumulator->get_string();
         } else if (!dls.dls_rows.empty()) {
+            lnav_data.ld_views[LNV_DB].reload_data();
+            lnav_data.ld_views[LNV_DB].set_left(0);
             if (lnav_data.ld_flags & LNF_HEADLESS) {
                 if (ec.ec_local_vars.size() == 1) {
                     ensure_view(&lnav_data.ld_views[LNV_DB]);
@@ -783,6 +785,17 @@ execute_from_file(exec_context& ec,
 Result<std::string, lnav::console::user_message>
 execute_any(exec_context& ec, const std::string& cmdline_with_mode)
 {
+    if (cmdline_with_mode.empty()) {
+        auto um = lnav::console::user_message::error("empty command")
+                      .with_help(
+                          "a command should start with ':', ';', '/', '|' and "
+                          "followed by the operation to perform");
+        if (!ec.ec_source.empty()) {
+            um.with_snippet(ec.ec_source.back());
+        }
+        return Err(um);
+    }
+
     std::string retval, alt_msg, cmdline = cmdline_with_mode.substr(1);
     auto _cleanup = finally([&ec] {
         if (ec.is_read_write() &&
@@ -942,7 +955,6 @@ sql_callback(exec_context& ec, sqlite3_stmt* stmt)
         return 0;
     }
 
-    auto& chart = dls.dls_chart;
     auto& vc = view_colors::singleton();
     int ncols = sqlite3_column_count(stmt);
     int row_number;
@@ -964,16 +976,17 @@ sql_callback(exec_context& ec, sqlite3_stmt* stmt)
 
             dls.push_header(colname, type, graphable);
             if (graphable) {
+                auto& hm = dls.dls_headers.back();
                 auto name_for_ident_attrs = colname;
                 auto attrs = vc.attrs_for_ident(name_for_ident_attrs);
                 for (size_t attempt = 0;
-                     chart.attrs_in_use(attrs) && attempt < 3;
+                     hm.hm_chart.attrs_in_use(attrs) && attempt < 3;
                      attempt++)
                 {
                     name_for_ident_attrs += " ";
                     attrs = vc.attrs_for_ident(name_for_ident_attrs);
                 }
-                chart.with_attrs_for_ident(colname, attrs);
+                hm.hm_chart.with_attrs_for_ident(colname, attrs);
                 dls.dls_headers.back().hm_title_attrs = attrs;
             }
         }
@@ -1052,6 +1065,8 @@ pipe_callback(exec_context& ec, const std::string& cmdline, auto_fd& fd)
             return std::string();
         });
     }
+    std::error_code errc;
+    ghc::filesystem::create_directories(lnav::paths::workdir(), errc);
     auto open_temp_res = lnav::filesystem::open_temp_file(lnav::paths::workdir()
                                                           / "exec.XXXXXX");
     if (open_temp_res.isErr()) {
@@ -1154,6 +1169,28 @@ exec_context::exec_context(logline_value_vector* line_values,
 void
 exec_context::execute(const std::string& cmdline)
 {
+    if (this->get_provenance<mouse_input>()) {
+        require(!lnav_data.ld_rl_view->is_active());
+
+        int context = 0;
+        switch (cmdline[0]) {
+            case '/':
+                context = lnav::enums::to_underlying(ln_mode_t::SEARCH);
+                break;
+            case ':':
+                context = lnav::enums::to_underlying(ln_mode_t::COMMAND);
+                break;
+            case ';':
+                context = lnav::enums::to_underlying(ln_mode_t::SQL);
+                break;
+            case '|':
+                context = lnav::enums::to_underlying(ln_mode_t::EXEC);
+                break;
+        }
+
+        lnav_data.ld_rl_view->append_to_history(context, cmdline.substr(1));
+    }
+
     auto exec_res = execute_any(*this, cmdline);
     if (exec_res.isErr()) {
         this->ec_error_callback_stack.back()(exec_res.unwrapErr());
