@@ -40,6 +40,76 @@
 
 using namespace std::chrono_literals;
 
+#    if !CURL_AT_LEAST_VERSION(7, 80, 0)
+extern "C"
+{
+const char*
+curl_url_strerror(CURLUcode error)
+{
+    switch (error) {
+        case CURLUE_OK:
+            return "No error";
+
+        case CURLUE_BAD_HANDLE:
+            return "An invalid CURLU pointer was passed as argument";
+
+        case CURLUE_BAD_PARTPOINTER:
+            return "An invalid 'part' argument was passed as argument";
+
+        case CURLUE_MALFORMED_INPUT:
+            return "Malformed input to a URL function";
+
+        case CURLUE_BAD_PORT_NUMBER:
+            return "Port number was not a decimal number between 0 and 65535";
+
+        case CURLUE_UNSUPPORTED_SCHEME:
+            return "Unsupported URL scheme";
+
+        case CURLUE_URLDECODE:
+            return "URL decode error, most likely because of rubbish in the "
+                   "input";
+
+        case CURLUE_OUT_OF_MEMORY:
+            return "A memory function failed";
+
+        case CURLUE_USER_NOT_ALLOWED:
+            return "Credentials was passed in the URL when prohibited";
+
+        case CURLUE_UNKNOWN_PART:
+            return "An unknown part ID was passed to a URL API function";
+
+        case CURLUE_NO_SCHEME:
+            return "No scheme part in the URL";
+
+        case CURLUE_NO_USER:
+            return "No user part in the URL";
+
+        case CURLUE_NO_PASSWORD:
+            return "No password part in the URL";
+
+        case CURLUE_NO_OPTIONS:
+            return "No options part in the URL";
+
+        case CURLUE_NO_HOST:
+            return "No host part in the URL";
+
+        case CURLUE_NO_PORT:
+            return "No port part in the URL";
+
+        case CURLUE_NO_QUERY:
+            return "No query part in the URL";
+
+        case CURLUE_NO_FRAGMENT:
+            return "No fragment part in the URL";
+    }
+
+    return "CURLUcode unknown";
+}
+}
+#    endif
+
+namespace {
+
 struct curl_request_eq {
     explicit curl_request_eq(const std::string& name) : cre_name(name){};
 
@@ -57,6 +127,8 @@ struct curl_request_eq {
     const std::string& cre_name;
 };
 
+}  // namespace
+
 int
 curl_request::debug_cb(
     CURL* handle, curl_infotype type, char* data, size_t size, void* userp)
@@ -102,6 +174,28 @@ curl_request::string_cb(void* data, size_t size, size_t nmemb, void* userp)
     return realsize;
 }
 
+long
+curl_request::complete(CURLcode result)
+{
+    double total_time = 0, download_size = 0, download_speed = 0;
+
+    this->cr_completions += 1;
+    curl_easy_getinfo(this->cr_handle, CURLINFO_TOTAL_TIME, &total_time);
+    log_debug("%s: total_time=%f", this->cr_name.c_str(), total_time);
+    curl_easy_getinfo(this->cr_handle, CURLINFO_SIZE_DOWNLOAD, &download_size);
+    log_debug("%s: download_size=%f", this->cr_name.c_str(), download_size);
+    curl_easy_getinfo(
+        this->cr_handle, CURLINFO_SPEED_DOWNLOAD, &download_speed);
+    log_debug("%s: download_speed=%f", this->cr_name.c_str(), download_speed);
+
+    return -1;
+}
+
+curl_looper::curl_looper() : cl_curl_multi(curl_multi_cleanup)
+{
+    this->cl_curl_multi.reset(curl_multi_init());
+}
+
 void
 curl_looper::loop_body()
 {
@@ -203,7 +297,8 @@ curl_looper::check_for_finished_requests()
     int msgs_left;
 
     while ((msg = curl_multi_info_read(this->cl_curl_multi, &msgs_left))
-           != nullptr) {
+           != nullptr)
+    {
         if (msg->msg != CURLMSG_DONE) {
             continue;
         }
