@@ -57,8 +57,10 @@
 #include "base/math_util.hh"
 #include "base/paths.hh"
 #include "fmtlib/fmt/format.h"
+#include "hasher.hh"
 #include "line_buffer.hh"
-#include "lnav_util.hh"
+#include "piper.looper.hh"
+#include "scn/scn.h"
 
 using namespace std::chrono_literals;
 
@@ -67,6 +69,10 @@ static const ssize_t DEFAULT_INCREMENT = 128 * 1024;
 static const ssize_t INITIAL_COMPRESSED_BUFFER_SIZE = 5 * 1024 * 1024;
 static const ssize_t MAX_COMPRESSED_BUFFER_SIZE = 32 * 1024 * 1024;
 
+const ssize_t line_buffer::DEFAULT_LINE_BUFFER_SIZE = 256 * 1024;
+const ssize_t line_buffer::MAX_LINE_BUFFER_SIZE
+    = 4 * 4 * line_buffer::DEFAULT_LINE_BUFFER_SIZE;
+
 class io_looper : public isc::service<io_looper> {};
 
 struct io_looper_tag {};
@@ -127,18 +133,11 @@ private:
 };
 /* XXX END */
 
-static int32_t
-read_le32(const unsigned char* data)
-{
-    return ((data[0] << 0) | (data[1] << 8) | (data[2] << 16)
-            | (data[3] << 24));
-}
-
 #define Z_BUFSIZE      65536U
 #define SYNCPOINT_SIZE (1024 * 1024)
 line_buffer::gz_indexed::gz_indexed()
 {
-    if ((this->inbuf = (Bytef*) malloc(Z_BUFSIZE)) == NULL) {
+    if ((this->inbuf = auto_mem<Bytef>::malloc(Z_BUFSIZE)) == NULL) {
         throw std::bad_alloc();
     }
 }
@@ -194,11 +193,69 @@ line_buffer::gz_indexed::continue_stream()
 }
 
 void
-line_buffer::gz_indexed::open(int fd)
+line_buffer::gz_indexed::open(int fd, lnav::gzip::header& hd)
 {
     this->close();
     this->init_stream();
     this->gz_fd = fd;
+
+    unsigned char name[1024];
+    unsigned char comment[4096];
+
+    name[0] = '\0';
+    comment[0] = '\0';
+
+    gz_header gz_hd;
+    memset(&gz_hd, 0, sizeof(gz_hd));
+    gz_hd.name = name;
+    gz_hd.name_max = sizeof(name);
+    gz_hd.comment = comment;
+    gz_hd.comm_max = sizeof(comment);
+
+    Bytef inbuf[8192];
+    Bytef outbuf[8192];
+    this->strm.next_out = outbuf;
+    this->strm.total_out = 0;
+    this->strm.avail_out = sizeof(outbuf);
+    this->strm.next_in = inbuf;
+    this->strm.total_in = 0;
+
+    if (inflateGetHeader(&this->strm, &gz_hd) == Z_OK) {
+        auto rc = pread(fd, inbuf, sizeof(inbuf), 0);
+        if (rc >= 0) {
+            this->strm.avail_in = rc;
+
+            inflate(&this->strm, Z_BLOCK);
+            inflateEnd(&this->strm);
+
+            this->strm.next_out = Z_NULL;
+            this->strm.next_in = Z_NULL;
+            this->strm.next_in = Z_NULL;
+            this->strm.total_in = 0;
+            this->strm.avail_in = 0;
+            this->init_stream();
+
+            switch (gz_hd.done) {
+                case 0:
+                    log_debug("%d: no gzip header data", fd);
+                    break;
+                case 1:
+                    hd.h_mtime.tv_sec = gz_hd.time;
+                    hd.h_name = std::string((char*) name);
+                    hd.h_comment = std::string((char*) comment);
+                    break;
+                default:
+                    log_error("%d: failed to read gzip header data", fd);
+                    break;
+            }
+        } else {
+            log_error("%d: failed to read gzip header from file: %s",
+                      fd,
+                      strerror(errno));
+        }
+    } else {
+        log_error("%d: unable to get gzip header", fd);
+    }
 }
 
 int
@@ -353,7 +410,33 @@ line_buffer::set_fd(auto_fd& fd)
             char gz_id[2 + 1 + 1 + 4];
 
             if (pread(fd, gz_id, sizeof(gz_id), 0) == sizeof(gz_id)) {
-                if (gz_id[0] == '\037' && gz_id[1] == '\213') {
+                auto piper_hdr_opt = lnav::piper::read_header(fd, gz_id);
+
+                if (piper_hdr_opt) {
+                    static intern_string_t SRC = intern_string::lookup("piper");
+
+                    auto meta_buf = std::move(piper_hdr_opt.value());
+
+                    auto meta_sf = string_fragment::from_bytes(meta_buf.in(),
+                                                               meta_buf.size());
+                    auto meta_parse_res
+                        = lnav::piper::header_handlers.parser_for(SRC).of(
+                            meta_sf);
+                    if (meta_parse_res.isErr()) {
+                        log_error("failed to parse piper header: %s",
+                                  meta_parse_res.unwrapErr()[0]
+                                      .to_attr_line()
+                                      .get_string()
+                                      .c_str());
+                        throw error(EINVAL);
+                    }
+
+                    this->lb_line_metadata = true;
+                    this->lb_file_offset
+                        = lnav::piper::HEADER_SIZE + meta_buf.size();
+                    this->lb_piper_header_size = this->lb_file_offset;
+                    this->lb_header = meta_parse_res.unwrap();
+                } else if (gz_id[0] == '\037' && gz_id[1] == '\213') {
                     int gzfd = dup(fd);
 
                     log_perror(fcntl(gzfd, F_SETFD, FD_CLOEXEC));
@@ -361,15 +444,19 @@ line_buffer::set_fd(auto_fd& fd)
                         close(gzfd);
                         throw error(errno);
                     }
-                    this->lb_gz_file.writeAccess()->open(gzfd);
+                    lnav::gzip::header hdr;
+
+                    this->lb_gz_file.writeAccess()->open(gzfd, hdr);
                     this->lb_compressed = true;
-                    this->lb_file_time
-                        = read_le32((const unsigned char*) &gz_id[4]);
+                    this->lb_file_time = hdr.h_mtime.tv_sec;
                     if (this->lb_file_time < 0) {
                         this->lb_file_time = 0;
                     }
                     this->lb_compressed_offset
                         = lseek(this->lb_fd, 0, SEEK_CUR);
+                    if (!hdr.empty()) {
+                        this->lb_header = std::move(hdr);
+                    }
                     this->resize_buffer(INITIAL_COMPRESSED_BUFFER_SIZE);
                 }
 #ifdef HAVE_BZLIB_H
@@ -404,7 +491,9 @@ line_buffer::set_fd(auto_fd& fd)
 void
 line_buffer::resize_buffer(size_t new_max)
 {
-    if (new_max > (size_t) this->lb_buffer.capacity()) {
+    if (new_max <= MAX_LINE_BUFFER_SIZE
+        && new_max > (size_t) this->lb_buffer.capacity())
+    {
         /* Still need more space, try a realloc. */
         this->lb_share_manager.invalidate_refs();
         this->lb_buffer.expand_to(new_max);
@@ -635,37 +724,19 @@ line_buffer::load_next_buffer()
     // log_debug("END preload read");
 
     if (start > this->lb_last_line_offset) {
-        auto* line_start = this->lb_alt_buffer.value().begin();
+        const auto* line_start = this->lb_alt_buffer.value().begin();
 
         do {
-            const char* msg = nullptr;
-            int faulty_bytes = 0;
-            bool valid_utf = true;
-            char* lf = nullptr;
-
             auto before = line_start - this->lb_alt_buffer->begin();
             auto remaining = this->lb_alt_buffer.value().size() - before;
-            auto utf8_end = is_utf8((unsigned char*) line_start,
-                                    remaining,
-                                    &msg,
-                                    &faulty_bytes,
-                                    '\n');
-            if (msg != nullptr) {
-                lf = (char*) memchr(line_start, '\n', remaining);
-                utf8_end = lf - line_start;
-                valid_utf = false;
-            }
-            if (utf8_end >= 0) {
-                lf = line_start + utf8_end;
-            }
+            auto frag = string_fragment::from_bytes(line_start, remaining);
+            auto utf_scan_res = is_utf8(frag, '\n');
+            auto lf = utf_scan_res.remaining_ptr(frag);
             this->lb_alt_line_starts.emplace_back(before);
-            this->lb_alt_line_is_utf.emplace_back(valid_utf);
+            this->lb_alt_line_is_utf.emplace_back(utf_scan_res.is_valid());
+            this->lb_alt_line_has_ansi.emplace_back(utf_scan_res.usr_has_ansi);
 
-            if (lf != nullptr) {
-                line_start = lf + 1;
-            } else {
-                line_start = nullptr;
-            }
+            line_start = lf;
         } while (line_start != nullptr
                  && line_start < this->lb_alt_buffer->end());
     }
@@ -721,9 +792,13 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         this->lb_alt_line_starts.clear();
         this->lb_line_is_utf = std::move(this->lb_alt_line_is_utf);
         this->lb_alt_line_is_utf.clear();
+        this->lb_line_has_ansi = std::move(this->lb_alt_line_has_ansi);
+        this->lb_alt_line_has_ansi.clear();
         this->lb_stats.s_used_preloads += 1;
     }
-    if (this->in_range(start) && this->in_range(start + max_length - 1)) {
+    if (this->in_range(start)
+        && (max_length == 0 || this->in_range(start + max_length - 1)))
+    {
         /* Cache already has the data, nothing to do. */
         retval = true;
         if (!lnav::pid::in_child && this->lb_seekable && this->lb_buffer.full()
@@ -982,11 +1057,16 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
 Result<line_info, std::string>
 line_buffer::load_next_line(file_range prev_line)
 {
+    const char* line_start = nullptr;
     bool done = false;
     line_info retval;
 
     require(this->lb_fd != -1);
 
+    if (this->lb_line_metadata && prev_line.fr_offset == 0) {
+        prev_line.fr_offset = this->lb_piper_header_size;
+    }
+
     auto offset = prev_line.next_offset();
     ssize_t request_size = INITIAL_REQUEST_SIZE;
     retval.li_file_range.fr_offset = offset;
@@ -1004,9 +1084,17 @@ line_buffer::load_next_line(file_range prev_line)
             return Ok(retval);
         }
     }
+    if (prev_line.next_offset() == 0) {
+        auto is_utf_res = is_utf8(string_fragment::from_bytes(
+            this->lb_buffer.begin(), this->lb_buffer.size()));
+        this->lb_is_utf8 = is_utf_res.is_valid();
+        if (!this->lb_is_utf8) {
+            log_warning("input is not utf8 -- %s", is_utf_res.usr_message);
+        }
+    }
     while (!done) {
         auto old_retval_size = retval.li_file_range.fr_size;
-        const char *line_start, *lf;
+        const char* lf = nullptr;
 
         /* Find the data in the cache and */
         line_start = this->get_range(offset, retval.li_file_range.fr_size);
@@ -1027,6 +1115,7 @@ line_buffer::load_next_line(file_range prev_line)
                 if (next_line_iter != this->lb_line_starts.end()) {
                     utf8_end = *next_line_iter - 1 - *start_iter;
                     found_in_cache = true;
+                    lf = line_start + utf8_end;
                 } else {
                     // log_debug("no next iter");
                 }
@@ -1036,26 +1125,14 @@ line_buffer::load_next_line(file_range prev_line)
         }
 
         if (!found_in_cache) {
-            const char* msg;
-            int faulty_bytes;
-
-            utf8_end = is_utf8((unsigned char*) line_start,
-                               retval.li_file_range.fr_size,
-                               &msg,
-                               &faulty_bytes,
-                               '\n');
-            if (msg != nullptr) {
-                lf = (char*) memchr(
-                    line_start, '\n', retval.li_file_range.fr_size);
-                utf8_end = lf - line_start;
-                retval.li_valid_utf = false;
+            auto frag = string_fragment::from_bytes(
+                line_start, retval.li_file_range.fr_size);
+            auto scan_res = is_utf8(frag, '\n');
+            lf = scan_res.remaining_ptr(frag);
+            if (lf != nullptr) {
+                lf -= 1;
             }
-        }
-
-        if (utf8_end >= 0) {
-            lf = line_start + utf8_end;
-        } else {
-            lf = nullptr;
+            retval.li_utf8_scan_result = scan_res;
         }
 
         auto got_new_data = old_retval_size != retval.li_file_range.fr_size;
@@ -1067,7 +1144,7 @@ line_buffer::load_next_line(file_range prev_line)
 #endif
         if (lf != nullptr
             || (retval.li_file_range.fr_size >= MAX_LINE_BUFFER_SIZE)
-            || (request_size == MAX_LINE_BUFFER_SIZE)
+            || (request_size >= MAX_LINE_BUFFER_SIZE)
             || (!got_new_data
                 && (!this->is_pipe() || request_size > DEFAULT_INCREMENT)))
         {
@@ -1122,7 +1199,9 @@ line_buffer::load_next_line(file_range prev_line)
             if (!this->is_pipe() || !this->is_pipe_closed()) {
                 retval.li_partial = true;
             }
-            request_size += DEFAULT_INCREMENT;
+            request_size
+                = std::min<ssize_t>(this->lb_buffer.size() + DEFAULT_INCREMENT,
+                                    MAX_LINE_BUFFER_SIZE);
         }
 
         if (!done
@@ -1141,16 +1220,45 @@ line_buffer::load_next_line(file_range prev_line)
               retval.li_file_range.fr_offset,
               (int) retval.li_partial);
 #endif
+
+    retval.li_file_range.fr_metadata.m_has_ansi
+        = retval.li_utf8_scan_result.usr_has_ansi;
+    retval.li_file_range.fr_metadata.m_valid_utf
+        = retval.li_utf8_scan_result.is_valid();
+
+    if (this->lb_line_metadata) {
+        auto sv = scn::string_view{
+            line_start,
+            (size_t) retval.li_file_range.fr_size,
+        };
+
+        char level;
+        auto scan_res = scn::scan(sv,
+                                  "{}.{}:{};",
+                                  retval.li_timestamp.tv_sec,
+                                  retval.li_timestamp.tv_usec,
+                                  level);
+        if (scan_res) {
+            retval.li_timestamp.tv_sec
+                = lnav::to_local_time(date::sys_seconds{std::chrono::seconds{
+                                          retval.li_timestamp.tv_sec}})
+                      .time_since_epoch()
+                      .count();
+            retval.li_level = abbrev2level(&level, 1);
+        }
+    }
+
     return Ok(retval);
 }
 
 Result<shared_buffer_ref, std::string>
-line_buffer::read_range(const file_range fr)
+line_buffer::read_range(file_range fr)
 {
     shared_buffer_ref retval;
     const char* line_start;
     file_ssize_t avail;
 
+#if 0
     if (this->lb_last_line_offset != -1
         && fr.fr_offset > this->lb_last_line_offset)
     {
@@ -1164,6 +1272,7 @@ line_buffer::read_range(const file_range fr)
                         fr.fr_offset,
                         this->lb_last_line_offset));
     }
+#endif
 
     if (!(this->in_range(fr.fr_offset)
           && this->in_range(fr.fr_offset + fr.fr_size - 1)))
@@ -1178,7 +1287,17 @@ line_buffer::read_range(const file_range fr)
         return Err(fmt::format(
             FMT_STRING("short-read (need: {}; avail: {})"), fr.fr_size, avail));
     }
+    if (this->lb_line_metadata) {
+        auto new_start
+            = static_cast<const char*>(memchr(line_start, ';', fr.fr_size));
+        if (new_start) {
+            auto offset = new_start - line_start + 1;
+            line_start += offset;
+            fr.fr_size -= offset;
+        }
+    }
     retval.share(this->lb_share_manager, line_start, fr.fr_size);
+    retval.get_metadata() = fr.fr_metadata;
 
     return Ok(std::move(retval));
 }
@@ -1294,7 +1413,7 @@ line_buffer::enable_cache()
         "%d:cache file path: %s", this->lb_fd.get(), cached_file_path.c_str());
 
     auto fl = lnav::filesystem::file_lock(cached_file_path);
-    auto guard = lnav::filesystem::file_lock::guard(fl);
+    auto guard = lnav::filesystem::file_lock::guard(&fl);
 
     if (ghc::filesystem::exists(cached_done_path)) {
         log_info("%d:using existing cache file");
@@ -1351,12 +1470,13 @@ line_buffer::cleanup_cache()
         auto now = std::chrono::system_clock::now();
         auto cache_path = line_buffer_cache_path();
         std::vector<ghc::filesystem::path> to_remove;
+        std::error_code ec;
 
         for (const auto& cache_subdir :
-             ghc::filesystem::directory_iterator(cache_path))
+             ghc::filesystem::directory_iterator(cache_path, ec))
         {
             for (const auto& entry :
-                 ghc::filesystem::directory_iterator(cache_subdir))
+                 ghc::filesystem::directory_iterator(cache_subdir, ec))
             {
                 auto mtime = ghc::filesystem::last_write_time(entry.path());
                 auto exp_time = mtime + 1h;
@@ -1370,7 +1490,7 @@ line_buffer::cleanup_cache()
 
         for (auto& entry : to_remove) {
             log_debug("removing compressed file cache: %s", entry.c_str());
-            ghc::filesystem::remove_all(entry);
+            ghc::filesystem::remove_all(entry, ec);
         }
     });
 }
