@@ -29,6 +29,7 @@
  * @file listview_curses.cc
  */
 
+#include <chrono>
 #include <cmath>
 
 #include "listview_curses.hh"
@@ -40,10 +41,33 @@
 #include "base/lnav_log.hh"
 #include "config.h"
 
+using namespace std::chrono_literals;
+
 list_gutter_source listview_curses::DEFAULT_GUTTER_SOURCE;
 
 listview_curses::listview_curses() : lv_scroll(noop_func{}) {}
 
+bool
+listview_curses::contains(int x, int y) const
+{
+    if (!this->vc_visible) {
+        return false;
+    }
+
+    if (view_curses::contains(x, y)) {
+        return true;
+    }
+
+    auto dim = this->get_dimensions();
+
+    if (this->vc_x <= x && x < this->vc_x + dim.second && this->vc_y <= y
+        && y < this->vc_y + dim.first)
+    {
+        return true;
+    }
+    return false;
+}
+
 void
 listview_curses::update_top_from_selection()
 {
@@ -60,26 +84,20 @@ listview_curses::update_top_from_selection()
         this->set_top(0_vl);
     } else if (this->lv_sync_selection_and_top) {
         this->set_top(this->lv_selection);
-    } else if (this->lv_selection == this->get_inner_height() - 1_vl) {
-        this->set_top(this->get_top_for_last_row());
     } else if (height <= this->lv_tail_space) {
         this->set_top(this->lv_selection);
-    } else if (this->lv_selection
-               >= (this->lv_top + height - this->lv_tail_space - 1_vl))
-    {
-        auto diff = this->lv_selection
-            - (this->lv_top + height - this->lv_tail_space - 1_vl);
+    } else if (this->lv_selection > (this->lv_top + height - 1_vl)) {
+        auto diff = this->lv_selection - (this->lv_top + height - 1_vl);
 
         if (height < 10 || diff < (height / 8_vl)) {
             // for small differences between the bottom and the
             // selection, just move a little bit.
-            this->set_top(
-                this->lv_selection - height + 1_vl + this->lv_tail_space, true);
+            this->set_top(this->lv_selection - height + 1_vl, true);
         } else {
             // for large differences, put the focus in the middle
             this->set_top(this->lv_selection - height / 2_vl, true);
         }
-    } else if (this->lv_selection <= this->lv_top) {
+    } else if (this->lv_selection < this->lv_top) {
         auto diff = this->lv_top - this->lv_selection;
 
         if (this->lv_selection > 0 && (height < 10 || diff < (height / 8_vl))) {
@@ -110,9 +128,10 @@ listview_curses::reload_data()
         }
         if (this->lv_selectable) {
             if (this->get_inner_height() == 0) {
-                this->set_selection(-1_vl);
+                this->set_selection_without_context(-1_vl);
             } else if (this->lv_selection >= this->get_inner_height()) {
-                this->set_selection(this->get_inner_height() - 1_vl);
+                this->set_selection_without_context(this->get_inner_height()
+                                                    - 1_vl);
             } else {
                 auto curr_sel = this->get_selection();
 
@@ -120,7 +139,7 @@ listview_curses::reload_data()
                     curr_sel = 0_vl;
                 }
                 this->lv_selection = -1_vl;
-                this->set_selection(curr_sel);
+                this->set_selection_without_context(curr_sel);
             }
 
             this->update_top_from_selection();
@@ -215,8 +234,9 @@ listview_curses::handle_key(int ch)
             {
                 this->set_selection(0_vl);
             } else {
-                this->shift_top(
-                    -(this->rows_available(this->lv_top, RD_UP) - 1_vl));
+                auto shift_amount
+                    = -(this->rows_available(this->lv_top, RD_UP) - 1_vl);
+                this->shift_top(shift_amount);
             }
             break;
 
@@ -232,8 +252,12 @@ listview_curses::handle_key(int ch)
                 }
             }
 
-            auto rows_avail
-                = this->rows_available(this->lv_top, RD_DOWN) - 1_vl;
+            auto rows_avail = this->rows_available(this->lv_top, RD_DOWN);
+            if (rows_avail == 0_vl) {
+                rows_avail = 2_vl;
+            } else if (rows_avail > 2_vl) {
+                rows_avail -= 1_vl;
+            }
             auto top_for_last = this->get_top_for_last_row();
 
             if ((this->lv_top < top_for_last)
@@ -243,6 +267,8 @@ listview_curses::handle_key(int ch)
                 if (this->lv_selection <= top_for_last) {
                     this->set_selection(top_for_last + 1_vl);
                 }
+            } else if (this->lv_top > top_for_last) {
+                this->set_top(top_for_last);
             } else {
                 this->shift_top(rows_avail);
 
@@ -364,34 +390,36 @@ listview_curses::get_overlay_top(vis_line_t row, size_t count, size_t total)
     return 0_vl;
 }
 
-void
+bool
 listview_curses::do_update()
 {
+    bool retval = false;
+
     if (this->lv_window == nullptr || this->lv_height == 0 || !this->vc_visible)
     {
-        view_curses::do_update();
-        return;
+        return view_curses::do_update();
     }
 
     std::vector<attr_line_t> row_overlay_content;
     vis_line_t height;
     unsigned long width;
 
-    this->update_top_from_selection();
     this->get_dimensions(height, width);
+    if (height <= 0) {
+        return retval;
+    }
+
+    this->update_top_from_selection();
     while (this->vc_needs_update) {
         auto& vc = view_colors::singleton();
         vis_line_t row;
         attr_line_t overlay_line;
         struct line_range lr;
         unsigned long wrap_width;
-        int y = this->lv_y, bottom;
+        int y = this->vc_y, bottom;
         auto role_attrs = vc.attrs_for_role(this->vc_default_role);
 
-        if (height <= 0) {
-            return;
-        }
-
+        retval = true;
         if (this->vc_width > 0) {
             width = std::min((unsigned long) this->vc_width, width);
         }
@@ -402,20 +430,21 @@ listview_curses::do_update()
         }
 
         size_t row_count = this->get_inner_height();
-        size_t blank_rows = 0;
         row = this->lv_top;
         bottom = y + height;
         std::vector<attr_line_t> rows(
             std::min((size_t) height, row_count - (int) this->lv_top));
         this->lv_source->listview_value_for_rows(*this, row, rows);
+        this->lv_display_lines.clear();
         while (y < bottom) {
             lr.lr_start = this->lv_left;
             lr.lr_end = this->lv_left + wrap_width;
             if (this->lv_overlay_source != nullptr
                 && this->lv_overlay_source->list_static_overlay(
-                    *this, y - this->lv_y, bottom - this->lv_y, overlay_line))
+                    *this, y - this->vc_y, bottom - this->vc_y, overlay_line))
             {
-                mvwattrline(this->lv_window, y, this->lv_x, overlay_line, lr);
+                this->lv_display_lines.push_back(static_overlay_content{});
+                mvwattrline(this->lv_window, y, this->vc_x, overlay_line, lr);
                 overlay_line.clear();
                 ++y;
             } else if (row < (int) row_count) {
@@ -425,31 +454,51 @@ listview_curses::do_update()
                     require_ge(attr.sa_range.lr_start, 0);
                 }
 
-                size_t remaining = 0;
+                this->lv_display_lines.push_back(main_content{row});
+                view_curses::mvwattrline_result write_res;
                 do {
-                    remaining = mvwattrline(this->lv_window,
+                    if (this->lv_word_wrap) {
+                        mvwhline(this->lv_window, y, this->vc_x, ' ', width);
+                    }
+                    write_res = mvwattrline(this->lv_window,
                                             y,
-                                            this->lv_x,
+                                            this->vc_x,
                                             al,
                                             lr,
                                             this->vc_default_role);
-                    if (this->lv_word_wrap) {
-                        mvwhline(this->lv_window,
-                                 y,
-                                 this->lv_x + wrap_width,
-                                 ' ',
-                                 width - wrap_width);
-                    }
-                    lr.lr_start += wrap_width;
-                    lr.lr_end += wrap_width;
+                    lr.lr_start = write_res.mr_chars_out;
+                    lr.lr_end = write_res.mr_chars_out + width - 1;
                     ++y;
-                } while (this->lv_word_wrap && y < bottom && remaining > 0);
+                } while (this->lv_word_wrap && y < bottom
+                         && write_res.mr_bytes_remaining > 0);
 
                 if (this->lv_overlay_source != nullptr) {
                     row_overlay_content.clear();
 
                     lr.lr_start = this->lv_left;
                     lr.lr_end = this->lv_left + wrap_width;
+
+                    auto ov_menu = this->lv_overlay_source->list_overlay_menu(
+                        *this, row);
+                    auto ov_menu_row = 0_vl;
+                    for (auto& ov_menu_line : ov_menu) {
+                        if (y >= bottom) {
+                            break;
+                        }
+
+                        this->lv_display_lines.push_back(overlay_menu{
+                            ov_menu_row,
+                        });
+                        mvwattrline(this->lv_window,
+                                    y,
+                                    this->vc_x,
+                                    ov_menu_line,
+                                    line_range{0, (int) wrap_width},
+                                    role_t::VCR_ALT_ROW);
+                        ov_menu_row += 1_vl;
+                        ++y;
+                    }
+
                     this->lv_overlay_source->list_value_for_overlay(
                         *this, row, row_overlay_content);
                     auto overlay_height = this->get_overlay_height(
@@ -467,9 +516,11 @@ listview_curses::do_update()
                             ov_hdr_attrs.ta_attrs |= A_UNDERLINE;
                             auto ov_hdr = hdr.value().with_attr_for_all(
                                 VC_STYLE.value(ov_hdr_attrs));
+                            this->lv_display_lines.push_back(
+                                static_overlay_content{});
                             mvwattrline(this->lv_window,
                                         y,
-                                        this->lv_x,
+                                        this->vc_x,
                                         ov_hdr,
                                         lr,
                                         role_t::VCR_STATUS_INFO);
@@ -486,9 +537,11 @@ listview_curses::do_update()
                             row_overlay_content[overlay_row].with_attr_for_all(
                                 VC_ROLE.value(role_t::VCR_CURSOR_LINE));
                         }
+                        this->lv_display_lines.push_back(
+                            overlay_content{row, overlay_row});
                         mvwattrline(this->lv_window,
                                     y,
-                                    this->lv_x,
+                                    this->vc_x,
                                     row_overlay_content[overlay_row],
                                     lr,
                                     role_t::VCR_ALT_ROW);
@@ -543,7 +596,7 @@ listview_curses::do_update()
                                       nullptr);
                             mvwaddch(this->lv_window,
                                      gutter_y,
-                                     this->lv_x + width - 2,
+                                     this->vc_x + width - 2,
                                      ch);
                         }
                     }
@@ -556,16 +609,14 @@ listview_curses::do_update()
                           vc.ensure_color_pair(role_attrs.ta_fg_color,
                                                role_attrs.ta_bg_color),
                           nullptr);
-                mvwhline(this->lv_window, y, this->lv_x, ' ', width);
+                this->lv_display_lines.push_back(empty_space{});
+                mvwhline(this->lv_window, y, this->vc_x, ' ', width);
                 ++y;
-                blank_rows += 1;
             }
         }
 
         if (this->lv_selectable && !this->lv_sync_selection_and_top
-            && this->lv_selection >= 0 && (row > this->lv_tail_space)
-            && (blank_rows < this->lv_tail_space)
-            && ((row - this->lv_tail_space) < this->lv_selection))
+            && this->lv_selection >= 0 && row < this->lv_selection)
         {
             this->shift_top(this->lv_selection - row + this->lv_tail_space);
             continue;
@@ -575,19 +626,18 @@ listview_curses::do_update()
             double progress = 1.0;
             double coverage = 1.0;
             double adjusted_height = (double) row_count / (double) height;
-            vis_line_t lines;
 
             if (row_count > 0) {
                 progress = (double) this->lv_top / (double) row_count;
                 coverage = (double) height / (double) row_count;
             }
 
-            y = this->lv_y + (int) (progress * (double) height);
-            lines = vis_line_t(
-                y + std::min((int) height, (int) (coverage * (double) height)));
+            this->lv_scroll_top = (int) (progress * (double) height);
+            this->lv_scroll_bottom = this->lv_scroll_top
+                + std::min((int) height, (int) (coverage * (double) height));
 
-            for (unsigned int gutter_y = this->lv_y;
-                 gutter_y < (this->lv_y + height);
+            for (unsigned int gutter_y = this->vc_y;
+                 gutter_y < (this->vc_y + height);
                  gutter_y++)
             {
                 int range_start = 0, range_end;
@@ -598,14 +648,14 @@ listview_curses::do_update()
 
                 if (row_count > 0) {
                     range_start
-                        = (double) (gutter_y - this->lv_y) * adjusted_height;
+                        = (double) (gutter_y - this->vc_y) * adjusted_height;
                 }
                 range_end = range_start + adjusted_height;
 
                 this->lv_gutter_source->listview_gutter_value_for_range(
                     *this, range_start, range_end, ch, role, bar_role);
-                if (gutter_y >= (unsigned int) y
-                    && gutter_y <= (unsigned int) lines)
+                if (gutter_y >= this->vc_y + this->lv_scroll_top
+                    && gutter_y <= this->vc_y + this->lv_scroll_bottom)
                 {
                     role = bar_role;
                 }
@@ -615,26 +665,28 @@ listview_curses::do_update()
                     attrs.ta_attrs,
                     vc.ensure_color_pair(attrs.ta_fg_color, attrs.ta_bg_color),
                     nullptr);
-                mvwaddch(this->lv_window, gutter_y, this->lv_x + width - 1, ch);
+                mvwaddch(this->lv_window, gutter_y, this->vc_x + width - 1, ch);
             }
-            wmove(this->lv_window, this->lv_y + height - 1, this->lv_x);
+            wmove(this->lv_window, this->vc_y + height - 1, this->vc_x);
         }
 
         if (this->lv_show_bottom_border) {
             cchar_t row_ch[width];
-            int y = this->lv_y + height - 1;
+            int bottom_y = this->vc_y + height - 1;
 
-            mvwin_wchnstr(this->lv_window, y, this->lv_x, row_ch, width - 1);
+            mvwin_wchnstr(
+                this->lv_window, bottom_y, this->vc_x, row_ch, width - 1);
             for (unsigned long lpc = 0; lpc < width - 1; lpc++) {
                 row_ch[lpc].attr |= A_UNDERLINE;
             }
-            mvwadd_wchnstr(this->lv_window, y, this->lv_x, row_ch, width - 1);
+            mvwadd_wchnstr(
+                this->lv_window, bottom_y, this->vc_x, row_ch, width - 1);
         }
 
         this->vc_needs_update = false;
     }
 
-    view_curses::do_update();
+    return view_curses::do_update() || retval;
 }
 
 void
@@ -736,16 +788,7 @@ listview_curses::shift_selection(shift_amount_t sa)
             {
                 this->set_top(top_for_last);
                 if (this->lv_selection <= top_for_last) {
-                    this->set_selection(top_for_last + 1_vl);
-                }
-            } else {
-                this->shift_top(rows_avail);
-
-                auto inner_height = this->get_inner_height();
-                if (this->lv_selectable && this->lv_top >= top_for_last
-                    && inner_height > 0_vl)
-                {
-                    this->set_selection(inner_height - 1_vl);
+                    new_selection = top_for_last + 1_vl;
                 }
             }
         }
@@ -764,6 +807,17 @@ scroll_polarity(mouse_button_t button)
 bool
 listview_curses::handle_mouse(mouse_event& me)
 {
+    auto GUTTER_REPEAT_DELAY
+        = std::chrono::duration_cast<std::chrono::microseconds>(100ms).count();
+
+    if (view_curses::handle_mouse(me)) {
+        return true;
+    }
+
+    if (!this->vc_enabled) {
+        return false;
+    }
+
     vis_line_t inner_height, height;
     struct timeval diff;
     unsigned long width;
@@ -774,50 +828,68 @@ listview_curses::handle_mouse(mouse_event& me)
 
     switch (me.me_button) {
         case mouse_button_t::BUTTON_SCROLL_UP:
-        case mouse_button_t::BUTTON_SCROLL_DOWN:
-            if (diff.tv_sec > 0 || diff.tv_usec > 80000) {
-                this->lv_scroll_accel = 1;
-                this->lv_scroll_velo = 0;
-            } else {
-                this->lv_scroll_accel += 2;
-            }
-            this->lv_scroll_velo += this->lv_scroll_accel;
-
-            this->shift_top(vis_line_t(scroll_polarity(me.me_button)
-                                       * this->lv_scroll_velo),
+        case mouse_button_t::BUTTON_SCROLL_DOWN: {
+            this->shift_top(vis_line_t(scroll_polarity(me.me_button) * 2_vl),
                             true);
-            break;
+            return true;
+        }
         default:
             break;
     }
-    this->lv_mouse_time = me.me_time;
-
     if (me.me_button != mouse_button_t::BUTTON_LEFT || inner_height == 0
-        || (this->lv_mouse_mode != lv_mode_t::DRAG
-            && me.me_x < (int) (width - 2)))
+        || (me.me_press_x < (int) (width - 2)))
     {
         return false;
     }
 
+    if (me.is_double_click_in(mouse_button_t::BUTTON_LEFT,
+                              line_range{(int) width - 2, (int) width}))
+    {
+        auto pct = (double) inner_height / (double) height;
+        auto new_top = (int) floor(((double) me.me_y * pct) + 0.5);
+        this->set_top(vis_line_t(new_top), true);
+        this->lv_mouse_mode = lv_mode_t::NONE;
+        return true;
+    }
+
+    switch (this->lv_mouse_mode) {
+        case lv_mode_t::NONE: {
+            if (me.me_x < (int) (width - 2)) {
+                return false;
+            }
+            break;
+        }
+        case lv_mode_t::DRAG:
+            break;
+        case lv_mode_t::UP:
+        case lv_mode_t::DOWN:
+            if (me.me_x < (int) (width - 2)) {
+                return true;
+            }
+            break;
+    }
+    if (me.me_state != mouse_button_state_t::BUTTON_STATE_RELEASED
+        && this->lv_mouse_mode != lv_mode_t::DRAG && diff.tv_sec == 0
+        && diff.tv_usec < GUTTER_REPEAT_DELAY)
+    {
+        return true;
+    }
+    this->lv_mouse_time = me.me_time;
+
     if (me.me_state == mouse_button_state_t::BUTTON_STATE_RELEASED) {
         this->lv_mouse_y = -1;
         this->lv_mouse_mode = lv_mode_t::NONE;
         return true;
     }
 
-    int scroll_top, scroll_bottom, shift_amount = 0, new_top = 0;
-    double top_pct, bot_pct, pct;
-
-    top_pct = (double) this->get_top() / (double) inner_height;
-    bot_pct = (double) this->get_bottom() / (double) inner_height;
-    scroll_top = (this->get_y() + (int) (top_pct * (double) height));
-    scroll_bottom = (this->get_y() + (int) (bot_pct * (double) height));
+    int shift_amount = 0;
 
     if (this->lv_mouse_mode == lv_mode_t::NONE) {
-        if ((scroll_top - 1) <= me.me_y && me.me_y <= (scroll_bottom + 1)) {
+        if (this->lv_scroll_top <= me.me_y && me.me_y <= this->lv_scroll_bottom)
+        {
             this->lv_mouse_mode = lv_mode_t::DRAG;
-            this->lv_mouse_y = me.me_y - scroll_top;
-        } else if (me.me_y < scroll_top) {
+            this->lv_mouse_y = me.me_y - this->lv_scroll_top;
+        } else if (me.me_y < this->lv_scroll_top) {
             this->lv_mouse_mode = lv_mode_t::UP;
         } else {
             this->lv_mouse_mode = lv_mode_t::DOWN;
@@ -830,27 +902,28 @@ listview_curses::handle_mouse(mouse_event& me)
             break;
 
         case lv_mode_t::UP:
-            if (me.me_y < scroll_top) {
+            if (me.me_y < this->lv_scroll_top) {
                 shift_amount = -1 * height;
             }
             break;
 
         case lv_mode_t::DOWN:
-            if (me.me_y > scroll_bottom) {
+            if (me.me_y > this->lv_scroll_bottom) {
                 shift_amount = height;
             }
             break;
 
-        case lv_mode_t::DRAG:
-            pct = (double) inner_height / (double) height;
-            new_top = me.me_y - this->get_y() - this->lv_mouse_y;
+        case lv_mode_t::DRAG: {
+            auto pct = (double) inner_height / (double) height;
+            auto new_top = me.me_y - this->lv_mouse_y;
             new_top = (int) floor(((double) new_top * pct) + 0.5);
-            this->set_top(vis_line_t(new_top));
+            this->set_top(vis_line_t(new_top), true);
             break;
+        }
     }
 
     if (shift_amount != 0) {
-        this->shift_top(vis_line_t(shift_amount));
+        this->shift_top(vis_line_t(shift_amount), true);
     }
 
     return true;
@@ -875,10 +948,10 @@ listview_curses::set_top(vis_line_t top, bool suppress_flash)
         this->lv_focused_overlay_selection = 0_vl;
         if (this->lv_selectable) {
             if (this->lv_selection < 0_vl) {
-                this->set_selection(top);
+                this->set_selection_without_context(top);
             } else if (this->lv_selection < top) {
                 auto sel_diff = this->lv_selection - old_top;
-                this->set_selection(top + sel_diff);
+                this->set_selection_without_context(top + sel_diff);
             } else {
                 auto sel_diff = this->lv_selection - old_top;
                 auto bot = this->get_bottom();
@@ -887,9 +960,15 @@ listview_curses::set_top(vis_line_t top, bool suppress_flash)
 
                 this->get_dimensions(height, width);
 
-                if (bot != -1_vl && (bot - top) >= (height - 1)) {
-                    if (this->lv_selection > (bot - this->lv_tail_space)) {
-                        this->set_selection(top + sel_diff);
+                if (bot == -1_vl) {
+                    this->set_selection_without_context(this->lv_top);
+                } else if (this->lv_selection < this->lv_top
+                           || bot < this->lv_selection)
+                {
+                    if (top + sel_diff > bot) {
+                        this->set_selection_without_context(bot);
+                    } else {
+                        this->set_selection_without_context(top + sel_diff);
                     }
                 }
             }
@@ -954,7 +1033,7 @@ listview_curses::rows_available(vis_line_t line,
 }
 
 void
-listview_curses::set_selection(vis_line_t sel)
+listview_curses::set_selection_without_context(vis_line_t sel)
 {
     if (this->lv_selectable) {
         if (this->lv_selection == sel) {
@@ -1013,6 +1092,23 @@ listview_curses::set_selection(vis_line_t sel)
     }
 }
 
+void
+listview_curses::set_selection(vis_line_t sel)
+{
+    this->set_selection_without_context(sel);
+
+    auto dim = this->get_dimensions();
+    if (this->lv_selection > 0 && this->lv_selection <= this->lv_top) {
+        this->set_top(this->lv_selection - 1_vl);
+    } else if (dim.first > this->lv_tail_space
+               && (this->lv_selection
+                   > (this->lv_top + (dim.first - 1_vl) - this->lv_tail_space)))
+    {
+        this->set_top(this->lv_selection + this->lv_tail_space
+                      - (dim.first - 1_vl));
+    }
+}
+
 vis_line_t
 listview_curses::get_top_for_last_row()
 {
@@ -1050,6 +1146,33 @@ listview_curses::shift_top(vis_line_t offset, bool suppress_flash)
     return this->lv_top;
 }
 
+void
+listview_curses::set_left(int left)
+{
+    if (this->lv_left == left || left < 0) {
+        return;
+    }
+
+    if (left > this->lv_left) {
+        unsigned long width;
+        vis_line_t height;
+
+        this->get_dimensions(height, width);
+        if (this->lv_show_scrollbar) {
+            width -= 1;
+        }
+        if ((this->get_inner_width() - this->lv_left) <= width) {
+            alerter::singleton().chime(
+                "the maximum width of the view has been reached");
+            return;
+        }
+    }
+
+    this->lv_left = left;
+    this->invoke_scroll();
+    this->set_needs_update();
+}
+
 size_t
 listview_curses::get_overlay_height(size_t total, vis_line_t view_height)
 {
@@ -1068,12 +1191,14 @@ listview_curses::set_overlay_selection(nonstd::optional<vis_line_t> sel)
         this->lv_overlay_source->list_value_for_overlay(
             *this, this->get_selection(), overlay_content);
         if (!overlay_content.empty()) {
-            if (sel.value() >= 0 && sel.value() < overlay_content.size()) {
-                this->lv_overlay_focused = true;
-                this->lv_focused_overlay_selection = sel.value();
-            } else {
-                this->lv_overlay_focused = true;
+            this->lv_overlay_focused = true;
+            if (sel.value() < 0) {
                 this->lv_focused_overlay_selection = 0_vl;
+            } else if (sel.value() >= overlay_content.size()) {
+                this->lv_focused_overlay_selection
+                    = vis_line_t(overlay_content.size());
+            } else {
+                this->lv_focused_overlay_selection = sel.value();
             }
         }
     } else {
