@@ -29,6 +29,7 @@
 
 #include "log_data_table.hh"
 
+#include "column_namer.hh"
 #include "config.h"
 #include "scn/scn.h"
 
@@ -102,8 +103,10 @@ log_data_table::get_columns_int()
                 collator = "naturalnocase";
                 break;
         }
-        metas.emplace_back(
-            intern_string::lookup(colname), kind, cols.size(), format.get());
+        metas.emplace_back(intern_string::lookup(colname),
+                           kind,
+                           logline_value_meta::table_column{cols.size()},
+                           format.get());
         cols.emplace_back(colname, sql_type, collator);
     }
     this->ldt_schema_id = dp.dp_schema_id;
@@ -119,7 +122,10 @@ log_data_table::next(log_cursor& lc, logfile_sub_source& lss)
     content_line_t cl;
 
     cl = lss.at(lc.lc_curr_line);
-    std::shared_ptr<logfile> lf = lss.find(cl);
+    auto* lf = lss.find_file_ptr(cl);
+    if (lf->get_format()->get_name() != this->ldt_format_impl->get_name()) {
+        return false;
+    }
     auto lf_iter = lf->begin() + cl;
 
     if (!lf_iter->is_message()) {
