@@ -215,6 +215,7 @@ log_vtab_impl::get_foreign_keys(std::vector<std::string>& keys_inout) const
     keys_inout.emplace_back("log_mark");
     keys_inout.emplace_back("log_time_msecs");
     keys_inout.emplace_back("log_top_line()");
+    keys_inout.emplace_back("log_msg_line()");
 }
 
 void
@@ -897,15 +898,19 @@ vt_column(sqlite3_vtab_cursor* cur, sqlite3_context* ctx, int col)
                     case log_footer_columns::path: {
                         const auto& fn = lf->get_filename();
 
-                        sqlite3_result_text(
-                            ctx, fn.c_str(), fn.length(), SQLITE_STATIC);
+                        sqlite3_result_text(ctx,
+                                            fn.c_str(),
+                                            fn.native().length(),
+                                            SQLITE_STATIC);
                         break;
                     }
                     case log_footer_columns::unique_path: {
                         const auto& fn = lf->get_unique_path();
 
-                        sqlite3_result_text(
-                            ctx, fn.c_str(), fn.length(), SQLITE_STATIC);
+                        sqlite3_result_text(ctx,
+                                            fn.c_str(),
+                                            fn.native().length(),
+                                            SQLITE_STATIC);
                         break;
                     }
                     case log_footer_columns::text: {
