@@ -414,6 +414,7 @@ readline_context::attempted_completion(const char* text, int start, int end)
         suggestion_possibilities.clear();
         suggestion_possibilities.emplace(rc_local_suggestion);
         arg_possibilities = &suggestion_possibilities;
+        rl_completion_append_character = 0;
     } else if (at_start
                && loaded_context->rc_possibilities.find(cmd_key)
                    != loaded_context->rc_possibilities.end())
@@ -680,8 +681,12 @@ readline_context::readline_context(std::string name,
     auto hpath = (config_dir / this->rc_name).string() + ".history";
     read_history(hpath.c_str());
     this->save();
+}
 
-    this->rc_append_character = ' ';
+void
+readline_context::set_history()
+{
+    history_set_history_state(&this->rc_history);
 }
 
 void
@@ -821,7 +826,7 @@ readline_curses::start()
     if (this->vc_width > 0) {
         ws.ws_col = this->vc_width;
     } else if (this->vc_width < 0) {
-        ws.ws_col -= this->vc_left;
+        ws.ws_col -= this->vc_x;
         ws.ws_col += this->vc_width;
     }
 
@@ -933,6 +938,8 @@ readline_curses::start()
 
     maxfd = std::max(STDIN_FILENO, this->rc_command_pipe[RCF_SLAVE].get());
 
+    static uint64_t last_h1, last_h2;
+
     while (looping) {
         fd_set ready_rfds;
         int rc;
@@ -951,8 +958,6 @@ readline_curses::start()
             }
         } else {
             if (FD_ISSET(STDIN_FILENO, &ready_rfds)) {
-                static uint64_t last_h1, last_h2;
-
                 struct itimerval itv;
 
                 itv.it_value.tv_sec = 0;
@@ -960,7 +965,6 @@ readline_curses::start()
                 itv.it_interval.tv_sec = 0;
                 itv.it_interval.tv_usec = 0;
                 setitimer(ITIMER_REAL, &itv, nullptr);
-
                 rl_callback_read_char();
                 if (RL_ISSTATE(RL_STATE_DONE) && !got_line) {
                     got_line = 1;
@@ -1008,14 +1012,17 @@ readline_curses::start()
 
                     if (h1 == last_h1 && h2 == last_h2) {
                         // do nothing
-                    } else if (sendcmd(this->rc_command_pipe[RCF_SLAVE],
-                                       complete_done ? 'l' : 'c',
-                                       rl_line_buffer,
-                                       rl_end)
-                               != 0)
-                    {
-                        perror("line: write failed");
-                        _exit(1);
+                    } else {
+                        rc_local_suggestion.clear();
+                        if (sendcmd(this->rc_command_pipe[RCF_SLAVE],
+                                    complete_done ? 'l' : 'c',
+                                    rl_line_buffer,
+                                    rl_end)
+                            != 0)
+                        {
+                            perror("line: write failed");
+                            _exit(1);
+                        }
                     }
                     last_h1 = h1;
                     last_h2 = h2;
@@ -1037,7 +1044,7 @@ readline_curses::start()
                 {
                     looping = false;
                 } else {
-                    int context, prompt_start = 0;
+                    int context, prompt_start = 0, new_point = 0;
                     char type[1024];
 
                     msg[rc] = '\0';
@@ -1047,6 +1054,18 @@ readline_curses::start()
                         log_perror(chdir(cwd));
                     } else if (startswith(msg, "sugg:")) {
                         rc_local_suggestion = &msg[5];
+                    } else if (sscanf(msg, "x:%d", &new_point) == 1) {
+                        if (rl_prompt) {
+                            new_point -= strlen(rl_prompt);
+                        }
+                        if (new_point < 0) {
+                            new_point = 0;
+                        }
+                        if (new_point > rl_end) {
+                            new_point = rl_end;
+                        }
+                        rl_point = new_point;
+                        rl_redisplay();
                     } else if (sscanf(msg, "i:%d:%n", &rl_point, &prompt_start)
                                == 1)
                     {
@@ -1152,6 +1171,11 @@ readline_curses::start()
 
                         this->rc_contexts[context]->rem_possibility(
                             std::string(type), std::string(&msg[prompt_start]));
+                    } else if (sscanf(msg, "ah:%d:%n", &context, &prompt_start)
+                               == 1)
+                    {
+                        this->rc_contexts[context]->set_history();
+                        add_history(&msg[prompt_start]);
                     } else if (sscanf(msg, "cpre:%d", &context) == 1) {
                         this->rc_contexts[context]->rc_prefixes.clear();
                     } else if (sscanf(msg, "cp:%d:%s", &context, type)) {
@@ -1284,6 +1308,23 @@ readline_curses::line_ready(const char* line)
     }
 }
 
+void
+readline_curses::append_to_history(int context, const std::string& line)
+{
+    if (line.empty()) {
+        return;
+    }
+
+    char buffer[2048];
+    snprintf(buffer, sizeof(buffer), "ah:%d:%s", context, line.c_str());
+    if (sendstring(
+            this->rc_command_pipe[RCF_MASTER], buffer, strlen(buffer) + 1)
+        == -1)
+    {
+        perror("add_possibility: write failed");
+    }
+}
+
 void
 readline_curses::check_poll_set(const std::vector<struct pollfd>& pollfds)
 {
@@ -1294,11 +1335,10 @@ readline_curses::check_poll_set(const std::vector<struct pollfd>& pollfds)
 
         rc = read(this->rc_pty[RCF_MASTER], buffer, sizeof(buffer));
         if (rc > 0) {
-            int old_x = this->vc_x;
+            int old_x = this->vc_cursor_x;
 
-            this->rc_suggestion.clear();
             this->map_output(buffer, rc);
-            if (this->vc_x != old_x) {
+            if (this->vc_cursor_x != old_x) {
                 this->rc_change(this);
             }
         }
@@ -1381,7 +1421,7 @@ readline_curses::check_poll_set(const std::vector<struct pollfd>& pollfds)
                         this->rc_blur(this);
                         break;
 
-                    case 'l':
+                    case 'l': {
                         this->rc_line_buffer = &msg[2];
                         if (this->rc_active_context != -1) {
                             this->rc_suggestion.clear();
@@ -1392,6 +1432,7 @@ readline_curses::check_poll_set(const std::vector<struct pollfd>& pollfds)
                             this->rc_display_match(this);
                         }
                         break;
+                    }
 
                     case 'c':
                         this->rc_line_buffer = &msg[2];
@@ -1451,7 +1492,7 @@ readline_curses::focus(int context,
     al.append(lnav::roles::suggestion(this->rc_suggestion));
     view_curses::mvwattrline(this->vc_window,
                              this->get_actual_y(),
-                             this->vc_left,
+                             this->vc_x,
                              al,
                              line_range{0, (int) this->get_actual_width()});
     if (!initial.empty()) {
@@ -1488,6 +1529,7 @@ readline_curses::set_suggestion(const std::string& value)
         perror("set_suggestion: write failed");
     }
     this->rc_suggestion = value;
+    this->set_needs_update();
 }
 
 void
@@ -1495,7 +1537,7 @@ readline_curses::abort()
 {
     char buffer[1024];
 
-    this->vc_x = 0;
+    this->vc_cursor_x = 0;
     snprintf(buffer, sizeof(buffer), "a");
     if (sendstring(this->rc_command_pipe[RCF_MASTER], buffer, strlen(buffer))
         == -1)
@@ -1600,11 +1642,11 @@ readline_curses::clear_possibilities(int context, std::string type)
     }
 }
 
-void
+bool
 readline_curses::do_update()
 {
     if (!this->vc_visible || this->vc_window == nullptr) {
-        return;
+        return false;
     }
 
     auto actual_width = this->get_actual_width();
@@ -1614,7 +1656,7 @@ readline_curses::do_update()
         attr_line_t alt_al;
         auto& vc = view_colors::singleton();
 
-        wmove(this->vc_window, this->get_actual_y(), this->vc_left);
+        wmove(this->vc_window, this->get_actual_y(), this->vc_x);
         auto attrs = vc.attrs_for_role(role_t::VCR_TEXT);
         wattr_set(this->vc_window,
                   attrs.ta_attrs,
@@ -1642,10 +1684,10 @@ readline_curses::do_update()
         lr.lr_end = this->rc_value.length();
         view_curses::mvwattrline(this->vc_window,
                                  this->get_actual_y(),
-                                 this->vc_left,
+                                 this->vc_x,
                                  this->rc_value,
                                  lr);
-        this->set_x(0);
+        this->set_cursor_x(0);
     }
 
     if (this->rc_active_context != -1) {
@@ -1654,24 +1696,46 @@ readline_curses::do_update()
         auto al = this->vc_line;
 
         if (hl != nullptr) {
-            hl(al, this->vc_left + this->vc_x);
+            hl(al, this->vc_x + this->vc_cursor_x);
         }
         al.append(lnav::roles::suggestion(this->rc_suggestion));
         view_curses::mvwattrline(this->vc_window,
                                  this->get_actual_y(),
-                                 this->vc_left,
+                                 this->vc_x,
                                  al,
                                  line_range{0, (int) actual_width});
 
-        wmove(
-            this->vc_window, this->get_actual_y(), this->vc_left + this->vc_x);
+        wmove(this->vc_window,
+              this->get_actual_y(),
+              this->vc_x + this->vc_cursor_x);
     }
+
+    return true;
+}
+
+bool
+readline_curses::handle_mouse(mouse_event& me)
+{
+    if (this->rc_active_context == -1) {
+        return false;
+    }
+
+    char buffer[32];
+
+    snprintf(buffer, sizeof(buffer), "x:%d", me.me_x);
+    if (sendstring(
+            this->rc_command_pipe[RCF_MASTER], buffer, strlen(buffer) + 1)
+        == -1)
+    {
+        perror("handle_mouse: write failed");
+    }
+    return true;
 }
 
 std::string
 readline_curses::get_match_string() const
 {
-    auto len = std::min((size_t) this->vc_x, this->rc_line_buffer.size())
+    auto len = std::min((size_t) this->vc_cursor_x, this->rc_line_buffer.size())
         - this->rc_match_start;
     auto* context = this->get_active_context();
 
@@ -1734,7 +1798,7 @@ readline_curses::window_change()
     if (this->vc_width > 0) {
         ws.ws_col = this->vc_width;
     } else if (this->vc_width < 0) {
-        ws.ws_col -= this->vc_left;
+        ws.ws_col -= this->vc_x;
         ws.ws_col += this->vc_width;
     }
     if (ioctl(this->rc_pty[RCF_MASTER], TIOCSWINSZ, &ws) == -1) {
