@@ -41,10 +41,6 @@
 #include "sql_util.hh"
 #include "view_curses.hh"
 
-static void readline_sqlite_highlighter_int(attr_line_t& al,
-                                            int x,
-                                            line_range sub);
-
 static bool
 is_bracket(const std::string& str, int index, bool is_lit)
 {
@@ -237,7 +233,7 @@ readline_command_highlighter(attr_line_t& al, int x)
         al, x, line_range{0, (int) al.get_string().length()});
 }
 
-static void
+void
 readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
 {
     static const char* brackets[] = {
@@ -280,9 +276,7 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
                    || attr.sa_type == &lnav::sql::PRQL_NUMBER_ATTR)
         {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_NUMBER));
-        } else if (attr.sa_type == &SQL_STRING_ATTR
-                   || attr.sa_type == &lnav::sql::PRQL_STRING_ATTR)
-        {
+        } else if (attr.sa_type == &SQL_STRING_ATTR) {
             if (lr.length() > 1 && al.al_string[lr.lr_end - 1] == '\'') {
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
             } else {
@@ -291,6 +285,8 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
                 alb.overlay_attr_for_char(lr.lr_start,
                                           VC_ROLE.value(role_t::VCR_ERROR));
             }
+        } else if (attr.sa_type == &lnav::sql::PRQL_STRING_ATTR) {
+            alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
         } else if (attr.sa_type == &SQL_OPERATOR_ATTR
                    || attr.sa_type == &lnav::sql::PRQL_OPERATOR_ATTR)
         {
