@@ -262,7 +262,7 @@ prql_cmd_from_prompt(exec_context& ec, const std::string& cmdline)
         = line_pair->first->get_format_ptr()->get_name().to_string();
     return {
         "",
-        lnav::prql::quote_ident(format_name),
+        lnav::prql::quote_ident(format_name) + " ",
     };
 }
 
@@ -485,7 +485,7 @@ static readline_context::command_t sql_commands[] = {
             .with_parameter({"table", "The table to use as a source"})
             .with_example({
                 "To pull data from the 'http_status_codes' database table",
-                "from db.http_status_codes | take 3",
+                "from http_status_codes | take 3",
                 help_example::language::prql,
             })
             .with_example({
@@ -667,6 +667,29 @@ static readline_context::command_t sql_commands[] = {
         "prql-source",
         {"prql-source"},
     },
+    {
+        "stats.hist",
+        prql_cmd_sort,
+        help_text("stats.hist", "Count the top values per bucket of time")
+            .prql_function()
+            .with_tags({"prql"})
+            .with_parameter(help_text{"col", "The column to count"})
+            .with_parameter(help_text{"slice", "The time slice"}
+                                .optional()
+                                .with_default_value("'1h'"))
+            .with_parameter(
+                help_text{"top", "The limit on the number of values to report"}
+                    .optional()
+                    .with_default_value("10"))
+            .with_example({
+                "To chart the values of ex_procname over time",
+                "from lnav_example_log | stats.hist ex_procname",
+                help_example::language::prql,
+            }),
+        nullptr,
+        "prql-source",
+        {"prql-source"},
+    },
     {
         "stats.sum_of",
         prql_cmd_sort,
