@@ -1384,10 +1384,6 @@ annotate_prql_statement(attr_line_t& al)
             lnav::pcre2pp::code::from_const(R"(\A(?:\[|\]|\{|\}|\(|\)))"),
             &PRQL_PAREN_ATTR,
         },
-        {
-            lnav::pcre2pp::code::from_const(R"(\A\|)"),
-            &PRQL_PIPE_ATTR,
-        },
         {
             lnav::pcre2pp::code::from(transform_re_str).unwrap(),
             &PRQL_TRANSFORM_ATTR,
@@ -1401,7 +1397,7 @@ annotate_prql_statement(attr_line_t& al)
             &PRQL_STRING_ATTR,
         },
         {
-            lnav::pcre2pp::code::from_const(R"(\A(?:f|r|s)?\"([^\"]|\\.)*\")"),
+            lnav::pcre2pp::code::from_const(R"(\A(?:f|r|s)?"([^\"]|\\.)*")"),
             &PRQL_STRING_ATTR,
         },
         {
@@ -1424,9 +1420,13 @@ annotate_prql_statement(attr_line_t& al)
         },
         {
             lnav::pcre2pp::code::from_const(
-                R"(\A(\*|\->{1,2}|<|>|=>|={1,2}|!|\-|\+|~=|\.\.|,))"),
+                R"(\A(\*|\->{1,2}|<|>|=>|={1,2}|\|\||&&|!|\-|\+|~=|\.\.|,|\?\?))"),
             &PRQL_OPERATOR_ATTR,
         },
+        {
+            lnav::pcre2pp::code::from_const(R"(\A\|)"),
+            &PRQL_PIPE_ATTR,
+        },
         {
             lnav::pcre2pp::code::from_const(R"(\A\.)"),
             &PRQL_DOT_ATTR,
