@@ -29,22 +29,28 @@
 
 #include "top_status_source.hh"
 
-#include <sqlite3.h>
-
-#include "base/injector.hh"
-#include "bound_tags.hh"
 #include "config.h"
 #include "lnav.hh"
-#include "lnav_config.hh"
-#include "logfile_sub_source.hh"
 #include "md2attr_line.hh"
 #include "md4cpp.hh"
 #include "shlex.hh"
-#include "shlex.resolver.hh"
-#include "sql_util.hh"
 #include "sqlitepp.client.hh"
+#include "top_status_source.cfg.hh"
 
-top_status_source::top_status_source()
+static const char* MSG_QUERY = R"(
+SELECT message FROM lnav_user_notifications
+  WHERE message IS NOT NULL AND
+        (expiration IS NULL OR expiration > datetime('now')) AND
+        (views IS NULL OR
+         json_contains(views, (SELECT name FROM lnav_top_view)))
+  ORDER BY priority DESC, expiration ASC
+  LIMIT 1
+)";
+
+top_status_source::top_status_source(auto_sqlite3& db,
+                                     const top_status_source_cfg& cfg)
+    : tss_config(cfg),
+      tss_user_msgs_stmt(prepare_stmt(db.in(), MSG_QUERY).unwrap())
 {
     this->tss_fields[TSF_TIME].set_width(28);
     this->tss_fields[TSF_TIME].set_role(role_t::VCR_STATUS_INFO);
@@ -58,12 +64,13 @@ top_status_source::update_time(const timeval& current_time)
 {
     auto& sf = this->tss_fields[TSF_TIME];
     char buffer[32];
+    tm current_tm;
 
     buffer[0] = ' ';
     strftime(&buffer[1],
              sizeof(buffer) - 1,
-             lnav_config.lc_ui_clock_format.c_str(),
-             localtime(&current_time.tv_sec));
+             this->tss_config.tssc_clock_format.c_str(),
+             localtime_r(&current_time.tv_sec, &current_tm));
     sf.set_value(buffer);
 }
 
@@ -76,47 +83,22 @@ top_status_source::update_time()
     this->update_time(tv);
 }
 
-static const char* MSG_QUERY = R"(
-SELECT message FROM lnav_user_notifications
-  WHERE message IS NOT NULL AND
-        (expiration IS NULL OR expiration > datetime('now')) AND
-        (views IS NULL OR
-         json_contains(views, (SELECT name FROM lnav_top_view)))
-  ORDER BY priority DESC
-  LIMIT 1
-)";
-
-struct user_msg_stmt {
-    user_msg_stmt()
-        : ums_stmt(
-            prepare_stmt(injector::get<auto_mem<sqlite3, sqlite_close_wrapper>&,
-                                       sqlite_db_tag>()
-                             .in(),
-                         MSG_QUERY)
-                .unwrap())
-    {
-    }
-
-    prepared_stmt ums_stmt;
-};
-
 void
 top_status_source::update_user_msg()
 {
-    static user_msg_stmt um_stmt;
-
     auto& al = this->tss_fields[TSF_USER_MSG].get_value();
     al.clear();
 
-    um_stmt.ums_stmt.reset();
-    auto fetch_res = um_stmt.ums_stmt.fetch_row<std::string>();
+    this->tss_user_msgs_stmt.reset();
+    auto fetch_res = this->tss_user_msgs_stmt.fetch_row<std::string>();
     fetch_res.match(
         [&al](const std::string& value) {
             shlex lexer(value);
             std::string user_note;
 
             lexer.with_ignore_quotes(true).eval(
-                user_note, lnav_data.ld_exec_context.ec_global_vars);
+                user_note,
+                scoped_resolver{&lnav_data.ld_exec_context.ec_global_vars});
 
             md2attr_line mdal;
             auto parse_res = md4cpp::parse(user_note, mdal);
