@@ -40,12 +40,14 @@
 #include "base/ansi_scrubber.hh"
 #include "base/attr_line.hh"
 #include "base/from_trait.hh"
+#include "base/injector.hh"
 #include "base/itertools.hh"
 #include "base/lnav_log.hh"
 #include "config.h"
 #include "lnav_config.hh"
 #include "shlex.hh"
 #include "view_curses.hh"
+#include "xterm_mouse.hh"
 
 #if defined HAVE_NCURSESW_CURSES_H
 #    include <ncursesw/term.h>
@@ -129,6 +131,91 @@ struct utf_to_display_adjustment {
     }
 };
 
+bool
+mouse_event::is_click_in(mouse_button_t button, int x_start, int x_end) const
+{
+    return this->me_button == button
+        && this->me_state == mouse_button_state_t::BUTTON_STATE_RELEASED
+        && (x_start <= this->me_x && this->me_x <= x_end)
+        && (x_start <= this->me_press_x && this->me_press_x <= x_end)
+        && this->me_y == this->me_press_y;
+}
+
+bool
+mouse_event::is_press_in(mouse_button_t button, line_range lr) const
+{
+    return this->me_button == button
+        && this->me_state == mouse_button_state_t::BUTTON_STATE_PRESSED
+        && lr.contains(this->me_x);
+}
+
+bool
+mouse_event::is_drag_in(mouse_button_t button, line_range lr) const
+{
+    return this->me_button == button
+        && this->me_state == mouse_button_state_t::BUTTON_STATE_DRAGGED
+        && lr.contains(this->me_press_x) && lr.contains(this->me_x);
+}
+
+bool
+mouse_event::is_double_click_in(mouse_button_t button, line_range lr) const
+{
+    return this->me_button == button
+        && this->me_state == mouse_button_state_t::BUTTON_STATE_DOUBLE_CLICK
+        && lr.contains(this->me_x) && this->me_y == this->me_press_y;
+}
+
+bool
+view_curses::handle_mouse(mouse_event& me)
+{
+    if (me.me_state != mouse_button_state_t::BUTTON_STATE_DRAGGED) {
+        this->vc_last_drag_child = nullptr;
+    }
+
+    for (auto* child : this->vc_children) {
+        auto x = this->vc_x + me.me_x;
+        auto y = this->vc_y + me.me_y;
+        if ((me.me_state == mouse_button_state_t::BUTTON_STATE_DRAGGED
+             && child == this->vc_last_drag_child && child->vc_x <= x
+             && x < (child->vc_x + child->vc_width))
+            || child->contains(x, y))
+        {
+            auto sub_me = me;
+
+            sub_me.me_x = x - child->vc_x;
+            sub_me.me_y = y - child->vc_y;
+            sub_me.me_press_x = this->vc_x + me.me_press_x - child->vc_x;
+            sub_me.me_press_y = this->vc_y + me.me_press_y - child->vc_y;
+            if (me.me_state == mouse_button_state_t::BUTTON_STATE_DRAGGED) {
+                this->vc_last_drag_child = child;
+            }
+            return child->handle_mouse(sub_me);
+        }
+    }
+    return false;
+}
+
+bool
+view_curses::contains(int x, int y) const
+{
+    if (!this->vc_visible) {
+        return false;
+    }
+
+    for (auto* child : this->vc_children) {
+        if (child->contains(x, y)) {
+            return true;
+        }
+    }
+    if (this->vc_x <= x
+        && (this->vc_width < 0 || x < this->vc_x + this->vc_width)
+        && this->vc_y == y)
+    {
+        return true;
+    }
+    return false;
+}
+
 void
 view_curses::awaiting_user_input()
 {
@@ -140,7 +227,7 @@ view_curses::awaiting_user_input()
     }
 }
 
-size_t
+view_curses::mvwattrline_result
 view_curses::mvwattrline(WINDOW* window,
                          int y,
                          const int x,
@@ -149,12 +236,13 @@ view_curses::mvwattrline(WINDOW* window,
                          role_t base_role)
 {
     auto& sa = al.get_attrs();
-    auto& line = al.get_string();
+    const auto& line = al.get_string();
     std::vector<utf_to_display_adjustment> utf_adjustments;
     std::string full_line;
 
     require(lr_chars.lr_end >= 0);
 
+    mvwattrline_result retval;
     auto line_width_chars = lr_chars.length();
     std::string expanded_line;
     line_range lr_bytes;
@@ -168,6 +256,7 @@ view_curses::mvwattrline(WINDOW* window,
             lr_bytes.lr_start = exp_start_index;
         } else if (char_index == lr_chars.lr_end) {
             lr_bytes.lr_end = exp_start_index;
+            retval.mr_chars_out = char_index;
         }
 
         switch (ch) {
@@ -175,6 +264,13 @@ view_curses::mvwattrline(WINDOW* window,
                 do {
                     expanded_line.push_back(' ');
                     char_index += 1;
+                    if (char_index == lr_chars.lr_start) {
+                        lr_bytes.lr_start = expanded_line.size();
+                    }
+                    if (char_index == lr_chars.lr_end) {
+                        lr_bytes.lr_end = expanded_line.size();
+                        retval.mr_chars_out = char_index;
+                    }
                 } while (expanded_line.size() % 8);
                 utf_adjustments.emplace_back(
                     lpc, expanded_line.size() - exp_start_index - 1);
@@ -196,6 +292,13 @@ view_curses::mvwattrline(WINDOW* window,
                 lpc += 1;
                 break;
 
+            case '\x07':
+                expanded_line.append("\U0001F514");
+                utf_adjustments.emplace_back(lpc, -1);
+                char_index += 1;
+                lpc += 1;
+                break;
+
             case '\r':
             case '\n':
                 expanded_line.push_back(' ');
@@ -227,6 +330,10 @@ view_curses::mvwattrline(WINDOW* window,
                     }
                     auto wch = read_res.unwrap();
                     char_index += wcwidth(wch);
+                    if (lr_bytes.lr_end == -1 && char_index > lr_chars.lr_end) {
+                        lr_bytes.lr_end = exp_start_index;
+                        retval.mr_chars_out = char_index - wcwidth(wch);
+                    }
                 }
                 break;
             }
@@ -238,7 +345,10 @@ view_curses::mvwattrline(WINDOW* window,
     if (lr_bytes.lr_end == -1) {
         lr_bytes.lr_end = expanded_line.size();
     }
-    size_t retval = expanded_line.size() - lr_bytes.lr_end;
+    if (retval.mr_chars_out == 0) {
+        retval.mr_chars_out = char_index;
+    }
+    retval.mr_bytes_remaining = expanded_line.size() - lr_bytes.lr_end;
 
     full_line = expanded_line;
 
@@ -361,6 +471,12 @@ view_curses::mvwattrline(WINDOW* window,
             } else if (iter->sa_type == &VC_ROLE) {
                 auto role = iter->sa_value.get<role_t>();
                 attrs = vc.attrs_for_role(role);
+
+                if (role == role_t::VCR_SELECTED_TEXT) {
+                    retval.mr_selected_text
+                        = string_fragment::from_str(line).sub_range(
+                            iter->sa_range.lr_start, iter->sa_range.lr_end);
+                }
             } else if (iter->sa_type == &VC_ROLE_FG) {
                 auto role_attrs
                     = vc.attrs_for_role(iter->sa_value.get<role_t>());
@@ -430,18 +546,49 @@ view_curses::mvwattrline(WINDOW* window,
         short cur_fg, cur_bg;
         pair_content(cur_pair, &cur_fg, &cur_bg);
 
-        if (fg_color[lpc] != -1 && bg_color[lpc] == -1) {
+        auto desired_fg = fg_color[lpc] != -1 ? fg_color[lpc] : cur_fg;
+        auto desired_bg = bg_color[lpc] != -1 ? bg_color[lpc] : cur_bg;
+        if (desired_fg >= COLOR_BLACK && desired_fg <= COLOR_WHITE) {
+            desired_fg = vc.ansi_to_theme_color(desired_fg);
+        }
+        if (desired_bg >= COLOR_BLACK && desired_bg <= COLOR_WHITE) {
+            desired_bg = vc.ansi_to_theme_color(desired_bg);
+        }
+        if (desired_fg == desired_bg) {
+            if (desired_bg >= 0
+                && desired_bg
+                    < view_colors::vc_active_palette->tc_palette.size())
+            {
+                auto adjusted_color
+                    = view_colors::vc_active_palette->tc_palette[desired_bg]
+                          .xc_lab_color;
+                if (adjusted_color.lc_l < 50.0) {
+                    adjusted_color.lc_l += 50.0;
+                } else {
+                    adjusted_color.lc_l -= 50.0;
+                }
+                bg_color[lpc] = view_colors::vc_active_palette->match_color(
+                    adjusted_color);
+            }
+        } else if (fg_color[lpc] >= 0
+                   && fg_color[lpc]
+                       < view_colors::vc_active_palette->tc_palette.size()
+                   && bg_color[lpc] == -1
+                   && base_attrs.ta_bg_color.value_or(0) >= 0
+                   && base_attrs.ta_bg_color.value_or(0)
+                       < view_colors::vc_active_palette->tc_palette.size())
+        {
             const auto& fg_color_info
-                = view_colors::vc_active_palette->tc_palette[fg_color[lpc]];
+                = view_colors::vc_active_palette->tc_palette.at(fg_color[lpc]);
             const auto& bg_color_info
-                = view_colors::vc_active_palette
-                      ->tc_palette[base_attrs.ta_bg_color.value_or(0)];
+                = view_colors::vc_active_palette->tc_palette.at(
+                    base_attrs.ta_bg_color.value_or(0));
 
             if (!fg_color_info.xc_lab_color.sufficient_contrast(
                     bg_color_info.xc_lab_color))
             {
                 auto adjusted_color = bg_color_info.xc_lab_color;
-                adjusted_color.lc_l -= 40.0;
+                adjusted_color.lc_l = std::max(0.0, adjusted_color.lc_l - 40.0);
                 auto new_bg = view_colors::vc_active_palette->match_color(
                     adjusted_color);
                 for (int lpc2 = lpc; lpc2 < line_width_chars; lpc2++) {
@@ -512,9 +659,9 @@ static const std::string COLOR_NAMES[] = {
     "white",
 };
 
-class color_listener : public lnav_config_listener {
+class ui_listener : public lnav_config_listener {
 public:
-    color_listener() : lnav_config_listener(__FILE__) {}
+    ui_listener() : lnav_config_listener(__FILE__) {}
 
     void reload_config(error_reporter& reporter) override
     {
@@ -547,11 +694,16 @@ public:
 
         if (view_colors::initialized) {
             vc.init_roles(iter->second, reporter);
+
+            auto& mouse_i = injector::get<xterm_mouse&>();
+            mouse_i.set_enabled(check_experimental("mouse")
+                                || lnav_config.lc_mouse_mode
+                                    == lnav_mouse_mode::enabled);
         }
     }
 };
 
-static color_listener _COLOR_LISTENER;
+static ui_listener _UI_LISTENER;
 term_color_palette* view_colors::vc_active_palette;
 
 void
@@ -577,7 +729,7 @@ view_colors::init(bool headless)
         auto reporter
             = [](const void*, const lnav::console::user_message& um) {};
 
-        _COLOR_LISTENER.reload_config(reporter);
+        _UI_LISTENER.reload_config(reporter);
     }
 }
 
@@ -1005,6 +1157,8 @@ view_colors::init_roles(const lnav_theme& lt,
         = this->to_attrs(lt, lt.lt_style_sep_ref_acc, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SUGGESTION)]
         = this->to_attrs(lt, lt.lt_style_suggestion, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SELECTED_TEXT)]
+        = this->to_attrs(lt, lt.lt_style_selected_text, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_RE_SPECIAL)]
         = this->to_attrs(lt, lt.lt_style_re_special, reporter);
@@ -1148,21 +1302,17 @@ view_colors::color_for_ident(const char* str, size_t len) const
     auto index = crc32(1, (const Bytef*) str, len);
     int retval;
 
-    if (COLORS >= 256) {
-        if (str[0] == '#' && (len == 4 || len == 7)) {
-            auto fg_res
-                = styling::color_unit::from_str(string_fragment(str, 0, len));
-            if (fg_res.isOk()) {
-                return this->match_color(fg_res.unwrap());
-            }
+    if (str[0] == '#' && (len == 4 || len == 7)) {
+        auto fg_res
+            = styling::color_unit::from_str(string_fragment(str, 0, len));
+        if (fg_res.isOk()) {
+            return this->match_color(fg_res.unwrap());
         }
-
-        auto offset = index % HI_COLOR_COUNT;
-        retval = this->vc_highlight_colors[offset];
-    } else {
-        retval = -1;
     }
 
+    auto offset = index % HI_COLOR_COUNT;
+    retval = this->vc_highlight_colors[offset];
+
     return retval;
 }
 
