@@ -1,5 +1,6 @@
 #! /bin/bash
 
+export TZ=UTC
 export YES_COLOR=1
 
 # journald json log format is not working"
@@ -12,6 +13,11 @@ run_cap_test ${lnav_test} -n \
     -I ${test_dir} \
     ${test_dir}/logfile_json.json
 
+run_cap_test ${lnav_test} -n \
+    -I ${test_dir} \
+    -c ':filter-in up service' \
+    ${test_dir}/logfile_json.json
+
 # json log format is not working"
 run_cap_test ${lnav_test} -n -I ${test_dir} \
     -c ':switch-to-view pretty' \
@@ -98,12 +104,23 @@ run_cap_test ${lnav_test} -n \
     ${test_dir}/logfile_nested_json.json
 
 # json log3 format is not working"
-run_cap_test ${lnav_test} -n \
+run_cap_test env TZ=UTC ${lnav_test} -n \
     -I ${test_dir} \
     ${test_dir}/logfile_json3.json
 
 # json log3 format is not working"
-run_cap_test ${lnav_test} -n \
+run_cap_test env TZ=UTC ${lnav_test} -n \
+    -I ${test_dir} \
+    -c ';select * from json_log3' \
+    -c ':write-csv-to -' \
+    ${test_dir}/logfile_json3.json
+
+run_cap_test env TZ=America/New_York ${lnav_test} -n \
+    -I ${test_dir} \
+    ${test_dir}/logfile_json3.json
+
+# json log3 format is not working"
+run_cap_test env TZ=America/New_York ${lnav_test} -n \
     -I ${test_dir} \
     -c ';select * from json_log3' \
     -c ':write-csv-to -' \
@@ -120,3 +137,27 @@ run_cap_test ${lnav_test} -n \
     -d /tmp/lnav.err \
     -I ${test_dir} \
     ${test_dir}/logfile_invalid_json2.json
+
+run_cap_test ${lnav_test} -n \
+    -I ${test_dir} \
+    ${test_dir}/logfile_mixed_json2.json
+
+run_cap_test ${lnav_test} -n \
+    -I ${test_dir} \
+    ${test_dir}/logfile_json_subsec.json
+
+run_cap_test ${lnav_test} -n \
+    ${test_dir}/logfile_bunyan.0
+
+run_cap_test ${lnav_test} -n \
+    ${test_dir}/logfile_cloudflare.json
+
+run_cap_test ${lnav_test} -n \
+    -c ':show-fields RayID' \
+    ${test_dir}/logfile_cloudflare.json
+
+run_cap_test ${lnav_test} -n \
+    ${test_dir}/logfile_nextcloud.0
+
+run_cap_test ${lnav_test} -n \
+    ${test_dir}/gharchive_log.jsonl
