@@ -1,5 +1,6 @@
 #! /bin/bash
 
+export TZ=UTC
 echo ${top_srcdir}
 echo ${top_builddir}
 
@@ -9,6 +10,9 @@ printf '#Date:\t20\x800-2-02\n0\n' | run_cap_test \
 printf '000\n000\n#Fields: 0\n0\n#Fields: 0\n0' | run_cap_test \
     env TEST_COMMENT="invalid w3c log" ${lnav_test} -n
 
+printf '#Date:\t3/9/3/0\x85 2\n0\n' | run_cap_test \
+    env TEST_COMMENT="invalid w3c timestamp" ${lnav_test} -n
+
 cat > rollover_in.0 <<EOF
 2600/2 0 00:00:00 0:
 00:2 0 00:00:00 0:
@@ -37,15 +41,6 @@ run_cap_test ${lnav_test} -n \
     -c ';SELECT * FROM logline' \
     ${test_dir}/logfile_block.1
 
-run_test ${lnav_test} -d /tmp/lnav.err -n -w logfile_stdin.0.log \
-    -c ':shexec sleep 1 && touch -t 200711030923 logfile_stdin.0.log' <<EOF
-2013-06-06T19:13:20.123  Hi
-EOF
-
-check_output "piping to stdin is not working?" <<EOF
-2013-06-06T19:13:20.123  Hi
-EOF
-
 if test x"${TSHARK_CMD}" != x""; then
   run_test env TZ=UTC ${lnav_test} -n ${test_dir}/dhcp.pcapng
 
@@ -54,12 +49,23 @@ if test x"${TSHARK_CMD}" != x""; then
 2004-12-05T19:16:24.317 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP Offer    - Transaction ID 0x3d1d
 2004-12-05T19:16:24.387 0.0.0.0 → 255.255.255.255 DHCP 314 DHCP Request  - Transaction ID 0x3d1e
 2004-12-05T19:16:24.387 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP ACK      - Transaction ID 0x3d1e
+EOF
+
+  # make sure piped binary data is left alone
+  run_test cat ${test_dir}/dhcp.pcapng | env TZ=UTC ${lnav_test} -n
+
+  check_output "pcap file is not recognized" <<EOF
+2004-12-05T19:16:24.317 0.0.0.0 → 255.255.255.255 DHCP 314 DHCP Discover - Transaction ID 0x3d1d
+2004-12-05T19:16:24.317 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP Offer    - Transaction ID 0x3d1d
+2004-12-05T19:16:24.387 0.0.0.0 → 255.255.255.255 DHCP 314 DHCP Request  - Transaction ID 0x3d1e
+2004-12-05T19:16:24.387 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP ACK      - Transaction ID 0x3d1e
 EOF
 
   run_test ${lnav_test} -n ${test_dir}/dhcp-trunc.pcapng
 
   check_error_output "truncated pcap file is not recognized" <<EOF
-error: unable to open file: {test_dir}/dhcp-trunc.pcapng -- tshark: The file "{test_dir}/dhcp-trunc.pcapng" appears to have been cut short in the middle of a packet.
+✘ error: unable to open file: {test_dir}/dhcp-trunc.pcapng
+ reason: tshark: The file "{test_dir}/dhcp-trunc.pcapng" appears to have been cut short in the middle of a packet.
 EOF
 fi
 
@@ -312,7 +318,7 @@ Nov 03 09:23:38 2007 -- 000
 Nov 03 09:47:02 2007 -- 000
 EOF
 
-touch -t 200711030923 ${srcdir}/logfile_syslog.1
+env TZ=UTC touch -t 200711030923 ${srcdir}/logfile_syslog.1
 run_test ./drive_logfile -t -f syslog_log ${srcdir}/logfile_syslog.1
 
 check_output "Syslog timestamp interpreted incorrectly for year end?" <<EOF
@@ -371,14 +377,12 @@ EOF
 run_test ${lnav_test} -n ${srcdir}/logfile_tcf.1
 
 check_output "timestamps with no dates are not rewritten?" <<EOF
-TCF 2014-04-06 09:59:47.191000: Server-Properties: {"Name":"TCF Protocol Logger","OSName":"Linux 3.2.0-60-generic","UserName":"xavier","AgentID":"1fde3dd1-d4be-4f79-8090-6f8d212f03bf","TransportName":"TCP","Proxy":"","ValueAdd":"1","Port":"1534"}
-TCF 2014-04-06 10:30:11.474000: 0: ---> C 2 RunControl getChildren null <eom>
-TCF 2014-04-06 11:01:11.475000: 0: <--- R 2  ["P1"] <eom>
+TCF 2014-04-06 09:59:47.191234: Server-Properties: {"Name":"TCF Protocol Logger","OSName":"Linux 3.2.0-60-generic","UserName":"xavier","AgentID":"1fde3dd1-d4be-4f79-8090-6f8d212f03bf","TransportName":"TCP","Proxy":"","ValueAdd":"1","Port":"1534"}
+TCF 2014-04-06 10:30:11.474442: 0: ---> C 2 RunControl getChildren null <eom>
+TCF 2014-04-06 11:01:11.475557: 0: <--- R 2  ["P1"] <eom>
 EOF
 
 
-# The TCSH format converts to local time, so we need to specify a TZ
-export TZ="UTC"
 run_test ./drive_logfile -t -f tcsh_history ${srcdir}/logfile_tcsh_history.0
 
 check_output "TCSH timestamp interpreted incorrectly?" <<EOF
@@ -396,6 +400,10 @@ Jul 20 22:59:29 2009 -- 000
 Jul 20 22:59:29 2009 -- 000
 EOF
 
+run_cap_test env TZ=America/Los_Angeles ./drive_logfile -t -f access_log ${srcdir}/logfile_access_log.0
+
+run_cap_test env TZ=America/Los_Angeles ${lnav_test} -n ${srcdir}/logfile_access_log.0
+
 run_test ./drive_logfile -t -f generic_log ${srcdir}/logfile_tai64n.0
 
 check_output "tai64n timestamps interpreted incorrectly?" <<EOF
@@ -543,6 +551,12 @@ error 0x0
 error 0x0
 EOF
 
+run_cap_test ./drive_logfile -t -f generic_log ${test_dir}/logfile_with_zones.0
+
+run_cap_test env TZ=America/Los_Angeles ./drive_logfile -t -f generic_log ${test_dir}/logfile_with_zones.0
+
+run_cap_test env TZ=America/New_York ./drive_logfile -t -f generic_log ${test_dir}/logfile_with_zones.0
+
 touch -t 200711030923 ${srcdir}/logfile_glog.0
 run_test ./drive_logfile -t -f glog_log ${srcdir}/logfile_glog.0
 
@@ -588,15 +602,6 @@ info 0x0
 error 0x0
 EOF
 
-run_test ${lnav_test} -d /tmp/lnav.err -nt -w logfile_stdin.log <<EOF
-Hi
-EOF
-
-check_output "piping to stdin is not working?" <<EOF
-2013-06-06T19:13:20.123  Hi
-2013-06-06T19:13:20.123  ---- END-OF-STDIN ----
-EOF
-
 run_test ${lnav_test} -C ${test_dir}/logfile_bad_access_log.0
 
 sed -ibak -e "s|/.*/logfile_bad_access_log.0|logfile_bad_access_log.0|g" `test_err_filename`
@@ -635,8 +640,8 @@ EOF
 run_test ${lnav_test} -n -I ${test_dir} ${srcdir}/logfile_epoch.0
 
 check_output "rewriting machine-oriented timestamp didn't work?" <<EOF
-2015-04-10 02:58:07.123000 Hello, World!
-2015-04-10 02:58:07.456000 Goodbye, World!
+2015-04-10 02:58:07.123 Hello, World!
+2015-04-10 02:58:07.456 Goodbye, World!
 EOF
 
 run_test ${lnav_test} -n -I ${test_dir} ${srcdir}/logfile_crlf.0
@@ -685,3 +690,49 @@ EOF
 # 192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkboot.gz HTTP/1.0" 404 46210 "-" "gPXE/0.9.7"
 # 192.168.202.254 - - [20/Jul/2009:22:59:29 +0000] "GET /vmw/vSphere/default/vmkernel.gz HTTP/1.0" 200 78929 "-" "gPXE/0.9.7"
 # EOF
+
+export YES_COLOR=1
+
+touch -t 202211030923 ${test_dir}/logfile_ansi.1
+
+run_cap_test ${lnav_test} -n \
+    -c ';SELECT log_time, log_body FROM syslog_log' \
+    ${test_dir}/logfile_ansi.1
+
+run_cap_test ${lnav_test} -n \
+    -c ':switch-to-view pretty' \
+    ${test_dir}/logfile_ansi.1
+
+run_cap_test ${lnav_test} -n \
+    -c ';SELECT basename(filepath),descriptor,mimetype,content FROM lnav_file_metadata' \
+    logfile_syslog.1.gz
+
+run_cap_test ${lnav_test} -n \
+    -c ':filter-in Air Mob' \
+    ${test_dir}/logfile_ansi.1
+
+export HOME="./file-tz"
+rm -rf "./file-tz"
+mkdir -p $HOME
+
+touch -t 200711030000 ${srcdir}/logfile_syslog.0
+
+run_cap_test ${lnav_test} -n \
+    -c ':set-file-timezone America/Los_Angeles' \
+    ${test_dir}/logfile_syslog.0
+
+# make sure the file options were saved and restored
+run_cap_test ${lnav_test} -n \
+    ${test_dir}/logfile_syslog.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT options_path, options FROM lnav_file" \
+    ${test_dir}/logfile_syslog.0
+
+run_cap_test ${lnav_test} -n \
+    -c ':set-file-timezone America/New_York' \
+    ${test_dir}/logfile_syslog.0
+
+run_cap_test ${lnav_test} -n \
+    -c ':set-file-timezone bad' \
+    ${test_dir}/logfile_syslog.0
