@@ -1,5 +1,7 @@
 #! /bin/bash
 
+export YES_COLOR=1
+
 run_cap_test ./drive_sql "select length(gzip(1))"
 
 run_cap_test ./drive_sql "select gunzip(gzip(1))"
@@ -68,6 +70,9 @@ run_cap_test ./drive_sql "select regexp_match('foo=(?<foo>\w+); (\w+)', 'foo=abc
 
 run_cap_test ./drive_sql "select regexp_match('foo=(?<foo>\w+); (\w+\.\w+)', 'foo=abc; 123.456') as result"
 
+run_cap_test ${lnav_test} -nN \
+   -c ";SELECT regexp_match('^(\w+)=([^;]+);', 'abc=def;ghi=jkl;')"
+
 run_cap_test ./drive_sql "select extract('foo=1') as result"
 
 run_cap_test ./drive_sql "select extract('foo=1; bar=2') as result"
@@ -106,9 +111,91 @@ run_cap_test ./drive_sql "SELECT encode('foo', null)"
 
 run_cap_test ./drive_sql "SELECT encode(null, 'base64')"
 
+run_cap_test ./drive_sql "SELECT encode('hi' || char(10), 'hex')"
+
 run_cap_test ./drive_sql "SELECT gunzip(decode(encode(gzip('Hello, World!'), 'base64'), 'base64'))"
 
+#run_cap_test env TEST_COMMENT=invalid_url ./drive_sql <<'EOF'
+#SELECT parse_url('https://bad@[fe::')
+#EOF
+
+run_cap_test env TEST_COMMENT=unsupported_url ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com:100000')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url1 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url2 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com/')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url3 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com/search?flag')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url4 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com/search?flag&flag2')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url5 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com/search?flag&flag2&=def')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url6 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com/sea%26rch?flag&flag2&=def#frag1%20space')
+EOF
+
+run_cap_test env TEST_COMMENT=parse_url7 ./drive_sql <<'EOF'
+SELECT parse_url('https://example.com/sea%26rch?flag&flag2&=def&flag3=abc+def#frag1%20space')
+EOF
+
+
+run_cap_test env TEST_COMMENT=unparse_url3 ./drive_sql <<'EOF'
+SELECT unparse_url(parse_url('https://example.com/search?flag'))
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url4 ./drive_sql <<'EOF'
+SELECT unparse_url(parse_url('https://example.com/search?flag&flag2'))
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url5 ./drive_sql <<'EOF'
+SELECT unparse_url(parse_url('https://example.com/search?flag&flag2&=def'))
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url6 ./drive_sql <<'EOF'
+SELECT unparse_url(parse_url('https://example.com/search?flag&flag2&=def#frag1%20space'))
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url7 ./drive_sql <<'EOF'
+SELECT unparse_url(NULL)
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url8 ./drive_sql <<'EOF'
+SELECT unparse_url(123)
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url9 ./drive_sql <<'EOF'
+SELECT unparse_url('[1, 2, 3]')
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url10 ./drive_sql <<'EOF'
+SELECT unparse_url(json_object('unknown', 'abc'))
+EOF
+
+run_cap_test env TEST_COMMENT=unparse_url11 ./drive_sql <<'EOF'
+SELECT unparse_url('{}')
+EOF
+
 run_cap_test ${lnav_test} -n \
     -c ';SELECT log_body, extract(log_body) from vmw_log' \
     -c ':write-json-to -' \
     ${test_dir}/logfile_vmw_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ';SELECT anonymize(bro_id_resp_h) FROM bro_http_log' \
+    ${test_dir}/logfile_bro_http.log.0
+
+run_cap_test ${lnav_test} -nN \
+    -c ";SELECT humanize_id('foo'), humanize_id('bar')"
